@@ -75,6 +75,7 @@ class T_Parms{
     int activeTest;
     int testCount;
     int uniqueID;
+    String ignoreErrors;
 };
 
 /** ErrorExit - Print out the error message as an
@@ -314,15 +315,6 @@ static void TestEnumerateClassNames (CIMClient* client, Boolean activeTest,
 
 }
 
-static void TestGetClass(CIMClient* client, Boolean activeTest,
-             Boolean verboseTest, String uniqueID)
-{
-    CIMClass c = client->getClass(
-        globalNamespace, CIMName ("CIM_ComputerSystem"), false, false, true);
-
-    XmlWriter::printClassElement(c);
-}
-
 /* This is both an active and passive class test and so uses the
     activetest variable
 */
@@ -546,8 +538,36 @@ static void TestQualifierOperations(CIMClient* client, Boolean activeTest,
 
 }
 
+static Boolean lookupError(const String& ignoreErrors, CIMStatusCode lookupCode)
+{
+    static const char ERROR_DELIMETER = ',';
+
+    Uint32 startPos = 0;
+    Uint32 strSize = ignoreErrors.size();
+
+    while (startPos < strSize)
+    {
+        Uint32 pos = ignoreErrors.find(startPos, ERROR_DELIMETER);
+        if (pos == PEG_NOT_FOUND)
+        {
+            pos = strSize; 
+        }
+
+        String errorStr = ignoreErrors.subString(startPos, pos - startPos);
+        if (errorStr == cimStatusCodeToString(lookupCode))
+        {
+            return true;
+        }
+
+        startPos = pos + 1;
+    }
+
+    return false;
+}
+
 static void TestInstanceGetOperations(CIMClient* client, Boolean activeTest,
-                                    Boolean verboseTest, String uniqueID)
+                                    Boolean verboseTest, String uniqueID,
+                                    String ignoreErrors)
 {
     // Get all instances
     // Get all classes
@@ -577,8 +597,10 @@ static void TestInstanceGetOperations(CIMClient* client, Boolean activeTest,
                instanceNames = 
                   client->enumerateInstanceNames(globalNamespace,classNames[i]);
                if (instanceNames.size() > 0)
-           cout << "Class " << classNames[i] << " "
-                << instanceNames.size() << " Instances" << endl;
+               {
+                   cout << "Class " << classNames[i] << " "
+                        << instanceNames.size() << " Instances" << endl;
+               }
            }
            catch(CIMException& e)
            {
@@ -586,6 +608,12 @@ static void TestInstanceGetOperations(CIMClient* client, Boolean activeTest,
                  {
                     numberOfNotSupportedClassesFound++;
                  }
+                 else if (lookupError(ignoreErrors, e.getCode()))
+                 {
+                     cerr << "CIMException : " << classNames[i].getString()
+                          << endl;
+                     cerr << e.getMessage() << endl;
+                 }
                  else
                  {
                     cerr << "CIMException : " << classNames[i] << endl;
@@ -991,80 +1019,6 @@ static void TestAssociationOperations(CIMClient* client, Boolean
     return;
 }
 
-/* Creates an instance with a method, executes the method and then deletes
-the instance
-
-Warning: This test works only as long as there is a provider, in this case, 
-it goes to the repository and gets dropped on the floor.
-*/
-
-static void TestMethodOperations( CIMClient* client, Boolean
-                     activeTest, Boolean verboseTest, String uniqueID)
-
-{
-    // Since the test modifies the repository, don't do it unless active set.
-    if (!activeTest)
-        return;
-
-    String name = uniqueID.append("_").append("TestSoftwarePkg");
-    CIMName cimName(name);
-    //Indication to be created
-    CIMClass cimClass = client->getClass(globalNamespace, cimName, false);
-    CIMInstance cimInstance(cimName);
-    cimInstance.addProperty(CIMProperty(CIMName ("PkgName"), String("WBEM")));
-    cimInstance.addProperty(CIMProperty(CIMName ("PkgIndex"), Uint32(101)));
-    cimInstance.addProperty(CIMProperty(CIMName ("trapOid"),
-        String("1.3.6.1.4.1.11.2.3.1.7.0.4")));
-    cimInstance.addProperty(CIMProperty(CIMName ("computerName"),
-        String("NU744781")));
-    CIMObjectPath instanceName = cimInstance.buildPath(cimClass);
-    instanceName.setNameSpace(globalNamespace);
-    client->createInstance(globalNamespace, cimInstance);
-
-    try
-    {
-        Array<CIMParamValue> inParams;
-        Array<CIMParamValue> outParams;
-        inParams.append(CIMParamValue("param1",
-            CIMValue(String("Hewlett-Packard"))));
-        inParams.append(CIMParamValue("param2",
-            CIMValue(String("California"))));
-            Uint32 testRepeat = 100;
-
-        for (Uint32 i = 0; i < testRepeat; i++)        // repeat the test x time
-            {
-                CIMValue retValue = client->invokeMethod(
-                    globalNamespace,
-                    instanceName,
-                    CIMName ("ChangeName"),
-                    inParams,
-                    outParams);
-                if (verboseTest)
-                {
-                        cout << "Output : " << retValue.toString() << endl;
-                        for (Uint8 i = 0; i < outParams.size(); i++)
-                            cout << outParams[i].getParameterName()
-                                << " : "
-                                << outParams[i].getValue().toString()
-                                << endl;
-                }
-            }
-            cout << "Executed " << testRepeat << " methods" << endl;
-    }
-    catch(ConnectionTimeoutException& e)
-    {
-        PEGASUS_STD(cerr) << "Warning: " << e.getMessage() << PEGASUS_STD(endl);
-        exit(2);
-    }
-    catch(Exception& e)
-    {
-        PEGASUS_STD(cerr) << "Error: " << e.getMessage() << PEGASUS_STD(endl);
-        exit(1);
-    }
-
-    client->deleteInstance(globalNamespace, instanceName);
-}
-
 /*
    Tests the invoke method request via the sample method provider.
 */
@@ -1297,7 +1251,11 @@ void GetOptions(
                          "Number of simultaneous client threads" },
 
         {"timeout","30000", false, Option::WHOLE_NUMBER, 0, 0, "to",
-                        "Client connection timeout value" }
+                        "Client connection timeout value" },
+
+        {"ignoreErrors", "", false, Option::STRING, 0, 0, "ignoreErrors",
+                         "Specifies a comma separated list of CIMError status"
+                         " code strings that should be ignored." }
 
     };
     const Uint32 NUM_OPTIONS = sizeof(optionsTable) / sizeof(optionsTable[0]);
@@ -1325,6 +1283,7 @@ ThreadReturnType PEGASUS_THREAD_CDECL executeTests(void *parm){
     Uint32 testCount = parms->testCount;
     Boolean verboseTest = parms->verboseTest == 0 ? false : true;
     Boolean activeTest = parms->activeTest == 0 ? false : true;
+    String ignoreErrors = parms->ignoreErrors;
     int id = parms->uniqueID;
     char id_[4];
     memset(id_,0x00,sizeof(id_));
@@ -1367,7 +1326,8 @@ ThreadReturnType PEGASUS_THREAD_CDECL executeTests(void *parm){
         testStart("Test Instance Get Operations");
         elapsedTime.reset();
         elapsedTime.start();
-        TestInstanceGetOperations(client,activeTest,verboseTest, uniqueID);
+        TestInstanceGetOperations(client,activeTest,verboseTest, uniqueID, 
+            ignoreErrors);
         elapsedTime.stop();
         testEnd(elapsedTime.getElapsed());
 
@@ -1385,15 +1345,6 @@ ThreadReturnType PEGASUS_THREAD_CDECL executeTests(void *parm){
         elapsedTime.stop();
         testEnd(elapsedTime.getElapsed());
 
-        /* Turn this one off until we get valid method to execute
-        testStart("Test Method Execution");
-        elapsedTime.reset();
-        elapsedTime.start();
-        TestMethodOperations(client, activeTest, verboseTest, uniqueID);
-        elapsedTime.stop();
-        testEnd(elapsedTime.getElapsed());
-        */
-
         testStart("Test Invoke Method Execution");
         elapsedTime.reset();
         elapsedTime.start();
@@ -1417,7 +1368,7 @@ ThreadReturnType PEGASUS_THREAD_CDECL executeTests(void *parm){
 }
 
 Thread * runTests(CIMClient *client, Uint32 testCount, Boolean activeTest, 
-        Boolean verboseTest, int uniqueID){
+        Boolean verboseTest, int uniqueID, String ignoreErrors){
         // package parameters, create thread and run...
         AutoPtr<T_Parms> parms(new T_Parms());
         parms->client.reset(client);
@@ -1425,6 +1376,7 @@ Thread * runTests(CIMClient *client, Uint32 testCount, Boolean activeTest,
         parms->activeTest = (activeTest) ? 1 : 0;
         parms->verboseTest = (verboseTest) ? 1 : 0;
         parms->uniqueID = uniqueID;
+        parms->ignoreErrors = ignoreErrors;
         AutoPtr<Thread> t(new Thread(executeTests, (void*)parms.release(), 
                     false));
 
@@ -1612,6 +1564,13 @@ int main(int argc, char** argv)
        cout << "password = " << password << endl;
     }
 
+    String ignoreErrors;
+    om.lookupValue("ignoreErrors", ignoreErrors);
+    if (ignoreErrors != String::EMPTY)
+    {
+       cout << "ignoreErrors = " << ignoreErrors << endl;
+    }
+
     Boolean useSLP =  false;
     Boolean localConnection = (om.valueEquals("local", "true"))? true: false;
     Boolean useSSL =  om.isTrue("ssl");
@@ -1682,7 +1641,8 @@ int main(int argc, char** argv)
         for(Uint32 i=0; i< clientConnections.size(); i++)
         {
             clientThreads.append(runTests(clientConnections[i], 
-                        repeatTestCount, activeTest, verboseTest, i));
+                        repeatTestCount, activeTest, verboseTest, i,
+                        ignoreErrors));
         }
 
         for(Uint32 i=0; i< clientThreads.size(); i++)
