@@ -40,7 +40,6 @@
 #include <sys/wait.h>
 #include <unistd.h>
 #include <signal.h>
-#include <grp.h>
 #include "Parent.h"
 #include "Log.h"
 #include "Messages.h"
@@ -56,6 +55,7 @@
 #include "PasswordFile.h"
 #include "Policy.h"
 #include "Macro.h"
+#include "FileHandle.h"
 
 #if defined(PEGASUS_PAM_AUTHENTICATION)
 # include "PAMAuth.h"
@@ -191,7 +191,7 @@ static void HandleOpenFileRequest(int sock)
             case 'w':
                 fd = open(
                     request.path,
-                    O_WRONLY | O_CREAT | O_TRUNC,
+                    O_WRONLY | O_TRUNC,
                     permissions);
                 break;
 
@@ -199,14 +199,32 @@ static void HandleOpenFileRequest(int sock)
             {
                 fd = open(
                     request.path,
-                    O_WRONLY | O_CREAT | O_APPEND,
+                    O_WRONLY | O_APPEND,
                     permissions);
                 break;
             }
         }
-
+        /* If the open call fails with ENOENT errno,then create the file.
+        If the umask is set to a non default value,then the file will not 
+        get created with permissions specified in the open system call.
+        So set the permissions for the file explicitly using fchmod.
+        */
         if (fd == -1)
-            response.status = -1;
+        {
+            if (errno == ENOENT && (request.mode == 'w' || request.mode == 'a'))
+            {
+                fd = open(request.path,O_CREAT | O_WRONLY,permissions);
+                if (fchmod(fd,permissions) != 0)
+                {
+                    response.status = -1;
+                    close(fd);
+                }
+            }
+            else
+            {
+                response.status = -1;
+            }
+        }
     }
     while (0);
 
@@ -323,58 +341,33 @@ static void HandleStartProviderAgentRequest(int sock)
 
         if (pid == 0)
         {
-            struct rlimit rlim;
-            char arg1[32];
-            char arg2[32];
+            char toPipeArg[32];
+            char fromPipeArg[32];
+            /* The user context is set here; no need to do it in cimprovagt. */
+            const char* setUserContextFlag = "0";
 
             /* Close unused pipe descriptors: */
 
             close(to[1]);
             close(from[0]);
 
+            /* Redirect terminal I/O if required and not yet daemonized. */
+
+            if (globals.initCompletePipe != -1 && !globals.bindVerbose)
+            {
+                RedirectTerminalIO();
+            }
+
             /*
              * Close unused descriptors. Leave stdin, stdout, stderr, and the
              * child's pipe descriptors open.
              */
 
-            if (getrlimit(RLIMIT_NOFILE, &rlim) == 0)
-            {
-                int i;
-
-                for (i = 3; i < (int)rlim.rlim_cur; i++)
-                {
-                    if (i != to[0] && i != from[1])
-                        close(i);
-                }
-            }
+            CloseUnusedDescriptors(to[0], from[1]);
 
 #if !defined(PEGASUS_DISABLE_PROV_USERCTXT)
 
-            if ((int)getgid() != gid)
-            {
-                if (setgid((gid_t)gid) != 0)
-                {
-                    Log(LL_SEVERE, "setgid(%d) failed\n", gid);
-                    _exit(1);
-                }
-            }
-
-            if ((int)getuid() != uid)
-            {
-                if (initgroups(request.userName, gid) != 0)
-                {
-                    Log(LL_SEVERE, "initgroups(%s, %d) failed\n",
-                        request.userName,
-                        gid);
-                    _exit(1);
-                }
-
-                if (setuid((uid_t)uid) != 0)
-                {
-                    Log(LL_SEVERE, "setuid(%d) failed\n", uid);
-                    _exit(1);
-                }
-            }
+            SetUserContext(request.userName, uid, gid);
 
             Log(LL_TRACE, "starting %s on module %s as user %s",
                 path, request.module, request.userName);
@@ -383,17 +376,37 @@ static void HandleStartProviderAgentRequest(int sock)
 
             /* Exec the CIMPROVAGT program. */
 
-            sprintf(arg1, "%d", to[0]);
-            sprintf(arg2, "%d", from[1]);
+            sprintf(toPipeArg, "%d", to[0]);
+            sprintf(fromPipeArg, "%d", from[1]);
 
-            Log(LL_TRACE, "execl(%s, %s, %s, %s, %s)\n",
-                path, path, arg1, arg2, request.module);
+            Log(LL_TRACE, "execl(%s, %s, %s, %s, %s, %s, %s)\n",
+                path,
+                path,
+                setUserContextFlag,
+                toPipeArg,
+                fromPipeArg,
+                request.userName,
+                request.module);
 
             /* Flawfinder: ignore */
-            execl(path, path, arg1, arg2, request.module, (char*)0);
+            execl(
+                path,
+                path,
+                setUserContextFlag,
+                toPipeArg,
+                fromPipeArg,
+                request.userName,
+                request.module,
+                (char*)0);
 
-            Log(LL_SEVERE, "execl(%s, %s, %s, %s, %s): failed\n",
-                path, path, arg1, arg2, request.module);
+            Log(LL_SEVERE, "execl(%s, %s, %s, %s, %s, %s, %s): failed\n",
+                path,
+                path,
+                setUserContextFlag,
+                toPipeArg,
+                fromPipeArg,
+                request.userName,
+                request.module);
             _exit(1);
         }
     }
@@ -433,7 +446,7 @@ static void HandleStartProviderAgentRequest(int sock)
 **==============================================================================
 */
 
-static void HandleDaemonizeExecutorRequest(int sock, int bindVerbose)
+static void HandleDaemonizeExecutorRequest(int sock)
 {
     struct ExecutorDaemonizeExecutorResponse response;
     int pid;
@@ -442,70 +455,18 @@ static void HandleDaemonizeExecutorRequest(int sock, int bindVerbose)
 
     Log(LL_TRACE, "HandleDaemonizeExecutorRequest()");
 
-    /* Fork (parent exits; child continues) */
-    /* (Ensures we are not a session leader so that setsid() will succeed.) */
-
-    pid = fork();
-
-    if (pid < 0)
+    if (!globals.bindVerbose)
     {
-        response.status = -1;
-        Fatal(FL, "fork() failed");
+        RedirectTerminalIO();
     }
 
-    if (pid > 0)
-        _exit(0);
-
-    /* Become session leader (so that our child process will not be one) */
-
-    if (setsid() < 0)
+    if (globals.initCompletePipe != -1)
     {
-        response.status = -1;
-        Fatal(FL, "setsid() failed");
-    }
+        ssize_t result;
 
-    /* Ignore SIGHUP: */
-
-    signal(SIGHUP, SIG_IGN);
-
-    /* Ignore SIGCHLD: */
-
-    signal(SIGCHLD, SIG_IGN);
-
-    /* Fork again (so we are not a session leader because our parent is): */
-
-    pid = fork();
-
-    if (pid < 0)
-    {
-        response.status = -1;
-        Fatal(FL, "fork() failed");
-    }
-
-    if (pid > 0)
-        _exit(0);
-
-    /* Catch SIGTERM: */
-
-    signal(SIGTERM, _sigHandler);
-
-    /* Make root the current directory. */
-
-    chdir("/");
-
-    if (!bindVerbose)
-    {
-        /* Close these file descriptors (stdin, stdout, stderr). */
-
-        close(0);
-        close(1);
-        close(2);
-
-        /* Direct standard input, output, and error to /dev/null: */
-
-        open("/dev/null", O_RDONLY);
-        open("/dev/null", O_RDWR);
-        open("/dev/null", O_RDWR);
+        EXECUTOR_RESTART(write(globals.initCompletePipe, "\0", 1), result);
+        close(globals.initCompletePipe);
+        globals.initCompletePipe = -1;
     }
 
     response.status = 0;
@@ -688,16 +649,9 @@ static void HandleAuthenticatePasswordRequest(int sock)
     }
     while (0);
 
-    if (status != 0)
-    {
-        Log(LL_WARNING, "Basic authentication failed for username %s",
-            request.username);
-    }
-    else
-    {
-        Log(LL_TRACE, "Basic authentication succeeded for username %s",
-            request.username);
-    }
+    Log(LL_TRACE, "Basic authentication attempt: username = %s, "
+        "successful = %s", 
+        request.username, status == 0 ? "TRUE" : "FALSE" );
 
     /* Send response message. */
 
@@ -762,9 +716,6 @@ static void HandleValidateUserRequest(int sock)
 
 #endif /* !PEGASUS_PAM_AUTHENTICATION */
 
-    if (status != 0)
-        Log(LL_WARNING, "User validation failed on %s", request.username);
-
     /* Send response message. */
 
     response.status = status;
@@ -800,12 +751,7 @@ static void HandleChallengeLocalRequest(int sock)
 
     /* Send response message. */
 
-    if (response.status != 0)
-    {
-        Log(LL_WARNING, "Local authentication failed for user \"%s\"",
-            request.user);
-    }
-    else
+    if (response.status == 0)
     {
         Strlcpy(response.challenge, challenge, sizeof(response.challenge));
     }
@@ -839,13 +785,6 @@ static void HandleAuthenticateLocalRequest(int sock)
 
     status = FinishLocalAuthentication(request.challenge, request.response);
 
-    /* Log result. */
-
-    if (status != 0)
-    {
-        Log(LL_WARNING, "Local authentication failed");
-    }
-
     /* Send response. */
 
     response.status = status;
@@ -902,7 +841,7 @@ static void HandleUpdateLogLevelRequest(int sock)
 **==============================================================================
 */
 
-void Parent(int sock, int childPid, int bindVerbose)
+void Parent(int sock, int initCompletePipe, int childPid, int bindVerbose)
 {
     /* Handle Ctrl-C. */
 
@@ -925,6 +864,14 @@ void Parent(int sock, int childPid, int bindVerbose)
 
     globals.childPid = childPid;
 
+    /* Save initCompletePipe; it is used at daemonization. */
+
+    globals.initCompletePipe = initCompletePipe;
+
+    /* Save bindVerbose; it is used at daemonization and cimprovagt start. */
+
+    globals.bindVerbose = bindVerbose;
+
     /* Prepares socket into non-blocking I/O. */
 
     SetNonBlocking(sock);
@@ -969,7 +916,7 @@ void Parent(int sock, int childPid, int bindVerbose)
                 break;
 
             case EXECUTOR_DAEMONIZE_EXECUTOR_MESSAGE:
-                HandleDaemonizeExecutorRequest(sock, bindVerbose);
+                HandleDaemonizeExecutorRequest(sock);
                 break;
 
             case EXECUTOR_RENAME_FILE_MESSAGE:
