@@ -93,40 +93,43 @@ static String _escapeSpecialCharacters(const String& str)
     return result;
 }
 
-static void _BubbleSort(Array<CIMKeyBinding>& x)
+static int _compare(const void* p1, const void* p2)
 {
-    Uint32 n = x.size();
+    const CIMKeyBinding* kb1 = (const CIMKeyBinding*)p1;
+    const CIMKeyBinding* kb2 = (const CIMKeyBinding*)p2;
+
+    return String::compareNoCase(
+        kb1->getName().getString(),
+        kb2->getName().getString());
+}
+
+static void _Sort(Array<CIMKeyBinding>& x)
+{
+    CIMKeyBinding* data = (CIMKeyBinding*)x.getData();
+    Uint32 size = x.size();
 
     //
     //  If the key is a reference, the keys in the reference must also be
     //  sorted
     //
-    for (Uint32 k = 0; k < n ; k++)
-        if (x[k].getType () == CIMKeyBinding::REFERENCE)
-        {
-            CIMObjectPath tmp (x[k].getValue ());
-            Array <CIMKeyBinding> keyBindings = tmp.getKeyBindings ();
-            _BubbleSort (keyBindings);
-            tmp.setKeyBindings (keyBindings);
-            x[k].setValue (tmp.toString ());
-        }
-
-    if (n < 2)
-        return;
-
-    for (Uint32 i = 0; i < n - 1; i++)
+    for (Uint32 k = 0; k < size; k++)
     {
-        for (Uint32 j = 0; j < n - 1; j++)
+        CIMKeyBinding& kb = data[k];
+
+        if (kb.getType() == CIMKeyBinding::REFERENCE)
         {
-            if (String::compareNoCase(x[j].getName().getString(),
-                                      x[j+1].getName().getString()) > 0)
-            {
-                CIMKeyBinding t = x[j];
-                x[j] = x[j+1];
-                x[j+1] = t;
-            }
+            CIMObjectPath tmp(kb.getValue());
+            Array<CIMKeyBinding> keyBindings = tmp.getKeyBindings();
+            _Sort(keyBindings);
+            tmp.setKeyBindings(keyBindings);
+            kb.setValue(tmp.toString());
         }
     }
+
+    if (size < 2)
+        return;
+
+    qsort((void*)data, size, sizeof(CIMKeyBinding), _compare);
 }
 
 ////////////////////////////////////////////////////////////////////////////////
@@ -408,12 +411,12 @@ Boolean operator==(const CIMKeyBinding& x, const CIMKeyBinding& y)
 class CIMObjectPathRep
 {
 public:
-    CIMObjectPathRep()
+    CIMObjectPathRep(): _refCounter(1)
     {
     }
 
     CIMObjectPathRep(const CIMObjectPathRep& x)
-        : _host(x._host), _nameSpace(x._nameSpace),
+        : _refCounter(1), _host(x._host), _nameSpace(x._nameSpace),
         _className(x._className), _keyBindings(x._keyBindings)
     {
     }
@@ -423,7 +426,7 @@ public:
         const CIMNamespaceName& nameSpace,
         const CIMName& className,
         const Array<CIMKeyBinding>& keyBindings)
-        : _host(host), _nameSpace(nameSpace),
+        : _refCounter(1), _host(host), _nameSpace(nameSpace),
         _className(className), _keyBindings(keyBindings)
     {
     }
@@ -451,6 +454,9 @@ public:
         return addr.isValid();
     }
 
+    // reference counter as member to avoid
+    // virtual function resolution overhead
+    AtomicInt _refCounter;
     //
     // Contains port as well (e.g., myhost:1234).
     //
@@ -461,6 +467,18 @@ public:
     Array<CIMKeyBinding> _keyBindings;
 };
 
+template<class REP>
+inline void Ref(REP* rep)
+{
+        rep->_refCounter++;
+}
+
+template<class REP>
+inline void Unref(REP* rep)
+{
+    if (rep->_refCounter.decAndTestIfZero())
+        delete rep;
+}
 
 CIMObjectPath::CIMObjectPath()
 {
@@ -469,7 +487,8 @@ CIMObjectPath::CIMObjectPath()
 
 CIMObjectPath::CIMObjectPath(const CIMObjectPath& x)
 {
-    _rep = new CIMObjectPathRep(*x._rep);
+    _rep = x._rep;
+    Ref(_rep);
 }
 
 CIMObjectPath::CIMObjectPath(const String& objectName)
@@ -477,8 +496,9 @@ CIMObjectPath::CIMObjectPath(const String& objectName)
     // Test the objectName out to see if we get an exception
     CIMObjectPath tmpRef;
     tmpRef.set(objectName);
-
-    _rep = new CIMObjectPathRep(*tmpRef._rep);
+    
+    _rep = tmpRef._rep;
+    Ref(_rep);
 }
 
 CIMObjectPath::CIMObjectPath(
@@ -490,27 +510,60 @@ CIMObjectPath::CIMObjectPath(
     // Test the objectName out to see if we get an exception
     CIMObjectPath tmpRef;
     tmpRef.set(host, nameSpace, className, keyBindings);
-
-    _rep = new CIMObjectPathRep(*tmpRef._rep);
+    _rep = tmpRef._rep;
+    Ref(_rep);
 }
 
 CIMObjectPath::~CIMObjectPath()
 {
-    delete _rep;
+    Unref(_rep);
 }
 
 CIMObjectPath& CIMObjectPath::operator=(const CIMObjectPath& x)
 {
-    *_rep = *x._rep;
+    if (x._rep != _rep)
+    {
+        Unref(_rep);
+        _rep = x._rep;
+        Ref(_rep);
+    }
     return *this;
 }
 
+static inline CIMObjectPathRep* _copyOnWriteCIMObjectPathRep(
+    CIMObjectPathRep* rep)
+{
+    if (rep->_refCounter.get() > 1)
+    {
+        CIMObjectPathRep* tmpRep= new CIMObjectPathRep(*rep);
+        Unref(rep);
+        return tmpRep;
+    }
+    else
+    {
+        return rep;
+    }
+}
+
 void CIMObjectPath::clear()
 {
-    _rep->_host.clear();
-    _rep->_nameSpace.clear();
-    _rep->_className.clear();
-    _rep->_keyBindings.clear();
+    // If there is more than one reference
+    // remove reference and get a new shiny empty representation
+    if (_rep->_refCounter.get() > 1)
+    {
+        Unref(_rep);
+        _rep = new CIMObjectPathRep();
+    }
+    else
+    {
+        // If there is only one reference
+        // no need to copy the data, we own it
+        // just clear the fields
+        _rep->_host.clear();
+        _rep->_nameSpace.clear();
+        _rep->_className.clear();
+        _rep->_keyBindings.clear();
+    }
 }
 
 void CIMObjectPath::set(
@@ -519,10 +572,18 @@ void CIMObjectPath::set(
     const CIMName& className,
     const Array<CIMKeyBinding>& keyBindings)
 {
-   setHost(host);
-   setNameSpace(nameSpace);
-   setClassName(className);
-   setKeyBindings(keyBindings);
+    if ((host != String::EMPTY) && !CIMObjectPathRep::isValidHostname(host))
+    {
+        throw MalformedObjectNameException(host);
+    }
+    
+    _rep = _copyOnWriteCIMObjectPathRep(_rep);
+
+    _rep->_host.assign(host);
+    _rep->_nameSpace = nameSpace;
+    _rep->_className = className;
+    _rep->_keyBindings = keyBindings;
+    _Sort(_rep->_keyBindings);
 }
 
 Boolean _parseHostElement(
@@ -655,8 +716,7 @@ void _parseKeyBindingPairs(
 
             p++;
 
-            Array<Uint8> keyValueUTF8;
-            keyValueUTF8.reserveCapacity(128);
+            Buffer keyValueUTF8(128);
 
             while (*p && *p != '"')
             {
@@ -690,18 +750,26 @@ void _parseKeyBindingPairs(
              */
             type = CIMKeyBinding::STRING;
 
-            try
+            /* Performance shortcut will check for
+               equal sign instead of doing the full
+               CIMObjectPath creation and exception handling
+            */
+            if (strchr(keyValueUTF8.getData(), '='))
             {
-                CIMObjectPath testForPath(valueString);
-                if (testForPath.getKeyBindings().size() > 0)
+                // found an equal sign, high probability for a reference
+                try
                 {
-                    // We've found a reference value!
-                    type = CIMKeyBinding::REFERENCE;
+                    CIMObjectPath testForPath(valueString);
+                    if (testForPath.getKeyBindings().size() > 0)
+                    {
+                        // We've found a reference value!
+                        type = CIMKeyBinding::REFERENCE;
+                    }
+                }
+                catch (const Exception &)
+                {
+                    // Not a reference value; leave type as STRING
                 }
-            }
-            catch (const Exception &)
-            {
-                // Not a reference value; leave type as STRING
             }
         }
         else if (toupper(*p) == 'T' || toupper(*p) == 'F')
@@ -781,11 +849,13 @@ void _parseKeyBindingPairs(
         }
     }
 
-    _BubbleSort(keyBindings);
+    _Sort(keyBindings);
 }
 
 void CIMObjectPath::set(const String& objectName)
 {
+    // the clear automatically ensures
+    // we have our own copy of the representation
     clear();
 
     //--------------------------------------------------------------------------
@@ -844,6 +914,7 @@ void CIMObjectPath::set(const String& objectName)
 
 CIMObjectPath& CIMObjectPath::operator=(const String& objectName)
 {
+    // set will call clear, which will cause copyOnWrite if necessary
     set(objectName);
     return *this;
 }
@@ -855,10 +926,13 @@ const String& CIMObjectPath::getHost() const
 
 void CIMObjectPath::setHost(const String& host)
 {
-    if ((host != String::EMPTY) && !CIMObjectPathRep::isValidHostname(host))
+    if ((host != String::EMPTY) &&
+        (host != System::getHostName()) &&
+        !CIMObjectPathRep::isValidHostname(host))
     {
         throw MalformedObjectNameException(host);
     }
+    _rep = _copyOnWriteCIMObjectPathRep(_rep);
 
     _rep->_host = host;
 }
@@ -870,6 +944,7 @@ const CIMNamespaceName& CIMObjectPath::getNameSpace() const
 
 void CIMObjectPath::setNameSpace(const CIMNamespaceName& nameSpace)
 {
+    _rep = _copyOnWriteCIMObjectPathRep(_rep);
    _rep->_nameSpace = nameSpace;
 }
 
@@ -880,6 +955,7 @@ const CIMName& CIMObjectPath::getClassName() const
 
 void CIMObjectPath::setClassName(const CIMName& className)
 {
+    _rep = _copyOnWriteCIMObjectPathRep(_rep);
     _rep->_className = className;
 }
 
@@ -890,8 +966,9 @@ const Array<CIMKeyBinding>& CIMObjectPath::getKeyBindings() const
 
 void CIMObjectPath::setKeyBindings(const Array<CIMKeyBinding>& keyBindings)
 {
+    _rep = _copyOnWriteCIMObjectPathRep(_rep);
     _rep->_keyBindings = keyBindings;
-    _BubbleSort(_rep->_keyBindings);
+    _Sort(_rep->_keyBindings);
 }
 
 String CIMObjectPath::toString() const
@@ -963,7 +1040,8 @@ String CIMObjectPath::toString() const
 
 String CIMObjectPath::_toStringCanonical() const
 {
-    CIMObjectPath ref = *this;
+    CIMObjectPath ref;
+    *ref._rep = *this->_rep;
 
     // Normalize hostname by changing to lower case
     ref._rep->_host.toLower(); // ICU_TODO:
