@@ -59,8 +59,12 @@ CIMPropertyRep::CIMPropertyRep(
     _arraySize(x._arraySize),
     _referenceClassName(x._referenceClassName),
     _classOrigin(x._classOrigin),
-    _propagated(x._propagated)
+    _propagated(x._propagated),
+    _ownerCount(0)
 {
+    // Set the CIM name tag.
+    _nameTag = generateCIMNameTag(_name);
+
     if (propagateQualifiers)
         x._qualifiers.cloneTo(_qualifiers);
 }
@@ -75,7 +79,7 @@ CIMPropertyRep::CIMPropertyRep(
     :
     _name(name), _value(value), _arraySize(arraySize),
     _referenceClassName(referenceClassName), _classOrigin(classOrigin),
-    _propagated(propagated)
+    _propagated(propagated), _ownerCount(0)
 {
     // ensure name is not null
     if (name.isNull())
@@ -83,6 +87,9 @@ CIMPropertyRep::CIMPropertyRep(
         throw UninitializedObjectException();
     }
 
+    // Set the CIM name tag.
+    _nameTag = generateCIMNameTag(_name);
+
     if ((arraySize != 0) &&
         (!value.isArray() || value.getArraySize() != arraySize))
     {
@@ -122,7 +129,19 @@ void CIMPropertyRep::setName(const CIMName& name)
         throw UninitializedObjectException();
     }
 
+    if (_ownerCount != 0 && _name != name)
+    {
+        MessageLoaderParms parms(
+            "Common.CIMPropertyRep.CONTAINED_PROPERTY_NAMECHANGEDEXCEPTION",
+            "Attempted to change the name of a property"
+                " already in a container.");
+        throw Exception(parms);
+    }
+
     _name = name;
+
+    // Set the CIM name tag.
+    _nameTag = generateCIMNameTag(_name);
 }
 
 void CIMPropertyRep::setClassOrigin(const CIMName& classOrigin)
@@ -173,13 +192,17 @@ void CIMPropertyRep::resolve(
 
     // Test the reference class name against the inherited property
 #ifdef PEGASUS_EMBEDDED_INSTANCE_SUPPORT
-    Boolean isEmbeddedInst = (_value.getType() == CIMTYPE_INSTANCE);
-    if (_value.getType() == CIMTYPE_REFERENCE || isEmbeddedInst)
+    if (_value.getType() == CIMTYPE_REFERENCE || 
+        _value.getType() == CIMTYPE_INSTANCE)
+#else // !PEGASUS_EMBEDDED_INSTANCE_SUPPORT
+    if (_value.getType() == CIMTYPE_REFERENCE)
+#endif // end PEGASUS_EMBEDDED_INSTANCE_SUPPORT
     {
         CIMName inheritedClassName;
         Array<CIMName> classNames;
 
-        if (isEmbeddedInst)
+#ifdef PEGASUS_EMBEDDED_INSTANCE_SUPPORT
+        if (_value.getType() == CIMTYPE_INSTANCE)
         {
             Uint32 pos = inheritedProperty.findQualifier("EmbeddedInstance");
             if (pos != PEG_NOT_FOUND)
@@ -206,6 +229,7 @@ void CIMPropertyRep::resolve(
             }
         }
         else
+#endif // end PEGASUS_EMBEDDED_INSTANCE_SUPPORT
         {
             CIMName referenceClass;
             if (_referenceClassName.isNull())
@@ -270,7 +294,6 @@ void CIMPropertyRep::resolve(
             successTree.appendArray(traversalHistory);
         }
     }
-#endif // PEGASUS_EMBEDDED_INSTANCE_SUPPORT
 
     _qualifiers.resolve(
         declContext, nameSpace, scope, isInstancePart,
@@ -316,16 +339,19 @@ void CIMPropertyRep::toXml(Buffer& out) const
         {
             // If the property array type is CIMObject, then
             //     encode the property in CIM-XML as a string array with the
-            //     EMBEDDEDOBJECT attribute (there is not currently a CIM-XML
+            //     EmbeddedObject attribute (there is not currently a CIM-XML
             //     "object" datatype)
 
             Array<CIMObject> a;
             _value.get(a);
             out << STRLIT(" TYPE=\"string\"");
             // If the Embedded Object is an instance, always add the
-            // EMBEDDEDOBJECT attribute.
+            // EmbeddedObject attribute.
             if (a.size() > 0 && a[0].isInstance())
+            {
+                out << STRLIT(" EmbeddedObject=\"object\"");
                 out << STRLIT(" EMBEDDEDOBJECT=\"object\"");
+            }
 #ifndef PEGASUS_SNIA_INTEROP_COMPATIBILITY
             else
             {
@@ -357,17 +383,18 @@ void CIMPropertyRep::toXml(Buffer& out) const
         {
             // If the property array type is CIMInstance, then
             //   encode the property in CIM-XML as a string array with the
-            //   EMBEDDEDOBJECT attribute (there is not currently a CIM-XML
+            //   EmbeddedObject attribute (there is not currently a CIM-XML
             //   "instance" datatype)
 
             Array<CIMInstance> a;
             _value.get(a);
-            out << " TYPE=\"string\"";
+            out << STRLIT(" TYPE=\"string\"");
 
-            // add the EMBEDDEDOBJECT attribute
+            // add the EmbeddedObject attribute
             if (a.size() > 0)
             {
-                out << " EMBEDDEDOBJECT=\"instance\"";
+                out << STRLIT(" EmbeddedObject=\"instance\"");
+                out << STRLIT(" EMBEDDEDOBJECT=\"instance\"");
 
                 // Note that if the macro PEGASUS_SNIA_INTEROP_COMPATIBILITY is
                 // defined, then the EmbeddedInstance qualifier will be added
@@ -479,7 +506,7 @@ void CIMPropertyRep::toXml(Buffer& out) const
         {
             // If the property type is CIMObject, then
             //   encode the property in CIM-XML as a string with the
-            //   EMBEDDEDOBJECT attribute (there is not currently a CIM-XML
+            //   EmbeddedObject attribute (there is not currently a CIM-XML
             //   "object" datatype)
 
             CIMObject a;
@@ -487,9 +514,12 @@ void CIMPropertyRep::toXml(Buffer& out) const
             out << STRLIT(" TYPE=\"string\"");
 
             // If the Embedded Object is an instance, always add the
-            // EMBEDDEDOBJECT attribute.
+            // EmbeddedObject attribute.
             if (a.isInstance())
+            {
+                out << STRLIT(" EmbeddedObject=\"object\"");
                 out << STRLIT(" EMBEDDEDOBJECT=\"object\"");
+            }
             // Else the Embedded Object is a class, always add the
             // EmbeddedObject qualifier.
 #ifndef PEGASUS_SNIA_INTEROP_COMPATIBILITY
@@ -522,6 +552,7 @@ void CIMPropertyRep::toXml(Buffer& out) const
             CIMInstance a;
             _value.get(a);
             out << " TYPE=\"string\"";
+            out << " EmbeddedObject=\"instance\"";
             out << " EMBEDDEDOBJECT=\"instance\"";
 
 #ifdef PEGASUS_SNIA_INTEROP_COMPATIBILITY
