@@ -18,7 +18,7 @@
 // rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
 // sell copies of the Software, and to permit persons to whom the Software is
 // furnished to do so, subject to the following conditions:
-// 
+//
 // THE ABOVE COPYRIGHT NOTICE AND THIS PERMISSION NOTICE SHALL BE INCLUDED IN
 // ALL COPIES OR SUBSTANTIAL PORTIONS OF THE SOFTWARE. THE SOFTWARE IS PROVIDED
 // "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT
@@ -47,7 +47,7 @@
 **==============================================================================
 */
 
-/* 
+/*
  * Message Queue Names
  */
 
@@ -176,6 +176,9 @@
 #define PEGASUS_DEFAULT_SHUTDOWN_TIMEOUT_SECONDS_STRING "30"
 #define PEGASUS_DEFAULT_SOCKETWRITE_TIMEOUT_SECONDS 20
 #define PEGASUS_DEFAULT_SOCKETWRITE_TIMEOUT_SECONDS_STRING "20"
+#define PEGASUS_SSL_ACCEPT_TIMEOUT_SECONDS 20
+#define PEGASUS_PROVIDER_IDLE_TIMEOUT_SECONDS 300
+
 
 
 /*
@@ -300,7 +303,7 @@
 #  undef PEGASUS_DEFAULT_MESSAGE_SOURCE     /* Not defined */
 # elif defined(PEGASUS_PLATFORM_ZOS_ZSERIES_IBM)
 #  define PEGASUS_CIMSERVER_START_FILE      "/var/wbem/cimserver.pid"
-#  define PEGASUS_CIMSERVER_START_LOCK_FILE "/tmp/cimserver_start.lock"
+#  define PEGASUS_CIMSERVER_START_LOCK_FILE "/var/wbem/cimserver_start.lock"
 #  define PEGASUS_REPOSITORY_DIR            "/var/wbem/repository"
 #  define PEGASUS_CURRENT_CONFIG_FILE_PATH  "/etc/wbem/cimserver_current.conf"
 #  define PEGASUS_PLANNED_CONFIG_FILE_PATH  "/etc/wbem/cimserver_planned.conf"
@@ -333,23 +336,36 @@
 # endif
 #endif
 #else  // Not PEGASUS_USE_RELEASE_DIRS
-# define PEGASUS_CIMSERVER_START_FILE      "/tmp/cimserver_start.conf"
-# define PEGASUS_CIMSERVER_START_LOCK_FILE "/tmp/cimserver_start.lock"
+# if defined(PEGASUS_OS_LINUX)
+#  define PEGASUS_CIMSERVER_STARTFILES_DIR "/var/tmp"
+# else
+#  define PEGASUS_CIMSERVER_STARTFILES_DIR "/tmp"
+# endif
+# define PEGASUS_CIMSERVER_START_FILE      \
+    PEGASUS_CIMSERVER_STARTFILES_DIR "/cimserver_start.conf"
+# define PEGASUS_CIMSERVER_START_LOCK_FILE \
+    PEGASUS_CIMSERVER_STARTFILES_DIR "/cimserver_start.lock"
 # define PEGASUS_REPOSITORY_DIR            "repository"
 # define PEGASUS_CURRENT_CONFIG_FILE_PATH  "cimserver_current.conf"
 # define PEGASUS_PLANNED_CONFIG_FILE_PATH  "cimserver_planned.conf"
 # define PEGASUS_SSLCLIENT_CERTIFICATEFILE "client.pem"
 # define PEGASUS_SSLCLIENT_RANDOMFILE      "ssl.rnd"
 # define PEGASUS_SSLSERVER_RANDOMFILE      "cimserver.rnd"
-# define PEGASUS_LOCAL_AUTH_DIR            "/tmp"
-# define PEGASUS_LOCAL_DOMAIN_SOCKET_PATH  "/tmp/cimxml.socket"
+# define PEGASUS_LOCAL_AUTH_DIR            PEGASUS_CIMSERVER_STARTFILES_DIR
+# define PEGASUS_LOCAL_DOMAIN_SOCKET_PATH  \
+    PEGASUS_CIMSERVER_STARTFILES_DIR "/cimxml.socket"
 # define PEGASUS_PAM_STANDALONE_PROC_NAME  "bin/cimservera"
 # define PEGASUS_PROVIDER_AGENT_PROC_NAME  "bin/cimprovagt"
 # undef PEGASUS_DEFAULT_MESSAGE_SOURCE      /* Not defined */
 #endif
 
 /* Use the PID file as a semaphore for repository access */
+/* Use the configuration file instead of the PID file on Windows*/
+#ifdef PEGASUS_OS_TYPE_WINDOWS
+#define PEGASUS_REPOSITORY_LOCK_FILE PEGASUS_PLANNED_CONFIG_FILE_PATH
+#else
 #define PEGASUS_REPOSITORY_LOCK_FILE PEGASUS_CIMSERVER_START_FILE
+#endif
 
 /*
  * Miscellaneous Constants
@@ -396,6 +412,33 @@
 */
 #define PEGASUS_INSTANCEID_GLOBAL_PREFIX "PG"
 
+/* Constants defining the size of the hash table used in the OrderedSet
+   implementation. Specific classes have their own hash table size to
+   accomodate for amounts of probable members
+*/
+#define PEGASUS_PROPERTY_ORDEREDSET_HASHSIZE 32
+#define PEGASUS_QUALIFIER_ORDEREDSET_HASHSIZE 16
+#define PEGASUS_PARAMETER_ORDEREDSET_HASHSIZE 16
+#define PEGASUS_METHOD_ORDEREDSET_HASHSIZE 16
+
+/* defines a number value for an OrderedSet when either 
+   the index to a specific member is unknown, meaning not initialized
+   (PEGASUS_ORDEREDSET_INDEX_UNKNOWN)
+   of the specific member is not part of the list
+   (PEGASUS_ORDEREDSET_INDEX_NOTFOUND)   
+*/
+#define PEGASUS_ORDEREDSET_INDEX_NOTFOUND 0xFFFFFFFF
+#define PEGASUS_ORDEREDSET_INDEX_UNKNOWN 0xFFFFFFFE
+
+/*
+   Defines the maximum authenticated username length.
+*/
+#ifdef PEGASUS_OS_PASE
+#define PEGASUS_MAX_USER_NAME_LEN 10
+#else
+#define PEGASUS_MAX_USER_NAME_LEN 256
+#endif
+
 /*
 **==============================================================================
 **
@@ -453,7 +496,8 @@ PEGASUS_COMMON_LINKAGE
     extern const CIMName PEGASUS_CLASSNAME_FORMATTEDINDSUBSCRIPTION;
 PEGASUS_COMMON_LINKAGE extern const CIMName PEGASUS_CLASSNAME_INDHANDLER;
 PEGASUS_COMMON_LINKAGE extern const CIMName PEGASUS_CLASSNAME_LSTNRDST;
-PEGASUS_COMMON_LINKAGE extern const CIMName PEGASUS_CLASSNAME_INDHANDLER_CIMXML;
+PEGASUS_COMMON_LINKAGE 
+    extern const CIMName PEGASUS_CLASSNAME_INDHANDLER_CIMXML;
 PEGASUS_COMMON_LINKAGE extern const CIMName PEGASUS_CLASSNAME_LSTNRDST_CIMXML;
 PEGASUS_COMMON_LINKAGE extern const CIMName PEGASUS_CLASSNAME_INDHANDLER_SNMP;
 PEGASUS_COMMON_LINKAGE
@@ -513,6 +557,8 @@ PEGASUS_COMMON_LINKAGE
     extern const CIMName PEGASUS_CLASSNAME_CIM_ELEMENTCONFORMSTOPROFILE;
 PEGASUS_COMMON_LINKAGE
     extern const CIMName PEGASUS_CLASSNAME_PG_ELEMENTCONFORMSTOPROFILE;
+PEGASUS_COMMON_LINKAGE
+    extern const CIMName PEGASUS_CLASSNAME_PG_ELEMENTCONFORMSTOPROFILE_RP_RP;
 PEGASUS_COMMON_LINKAGE
     extern const CIMName PEGASUS_CLASSNAME_PG_SUBPROFILEREQUIRESPROFILE;
 PEGASUS_COMMON_LINKAGE
