@@ -131,7 +131,8 @@ HTTPAcceptor::HTTPAcceptor(Monitor* monitor,
      _connectionType(connectionType),
      _portNumber(portNumber),
      _sslcontext(sslcontext),
-     _sslContextObjectLock(sslContextObjectLock)
+     _sslContextObjectLock(sslContextObjectLock),
+     _idleConnectionTimeoutSeconds(0)
 {
    Socket::initializeInterface();
 
@@ -388,10 +389,11 @@ void HTTPAcceptor::_bind()
         MessageLoaderParms parms("Common.HTTPAcceptor.FAILED_CREATE_SOCKET",
             "Failed to create socket");
         PEG_TRACE_CSTRING(TRC_DISCARDED_DATA, Tracer::LEVEL2,
-            "HTTPAcceptor::_bind _rep->socket < 0");
+            "HTTPAcceptor::_bind:  createSocket() _rep->socket failed");
         throw BindFailedException(parms);
     }
 
+    Socket::disableBlocking(_rep->socket);
 
 // set the close-on-exec bit for this file handle.
 // any unix that forks needs this bit set.
@@ -440,11 +442,14 @@ void HTTPAcceptor::_bind()
     //
     if (::bind(_rep->socket, _rep->address, _rep->address_size) < 0)
     {
+        MessageLoaderParms parms(
+            "Common.HTTPAcceptor.FAILED_BIND_SOCKET_DETAIL",
+            "Failed to bind socket on port $0: $1.",
+            _portNumber, PEGASUS_SYSTEM_NETWORK_ERRORMSG_NLS);
+
         Socket::close(_rep->socket);
         delete _rep;
         _rep = 0;
-        MessageLoaderParms parms("Common.HTTPAcceptor.FAILED_BIND_SOCKET",
-            "Failed to bind socket");
         PEG_TRACE_CSTRING(TRC_DISCARDED_DATA, Tracer::LEVEL2,
             "HTTPAcceptor::_bind: Failed to bind socket.");
         throw BindFailedException(parms);
@@ -480,11 +485,15 @@ void HTTPAcceptor::_bind()
                 S_IRGRP | S_IWGRP | S_IXGRP |
                 S_IROTH | S_IWOTH | S_IXOTH ) < 0 )
         {
+            MessageLoaderParms parms(
+                "Common.HTTPAcceptor.FAILED_SET_LDS_FILE_OPTION",
+                "Failed to set permission on local domain socket {0}: {1}.",
+                PEGASUS_LOCAL_DOMAIN_SOCKET_PATH,
+                PEGASUS_SYSTEM_ERRORMSG_NLS );
+
             Socket::close(_rep->socket);
             delete _rep;
             _rep = 0;
-            MessageLoaderParms parms("Common.HTTPAcceptor.FAILED_BIND_SOCKET",
-                "Failed to bind socket");
             PEG_TRACE_CSTRING(TRC_DISCARDED_DATA, Tracer::LEVEL2,
                 "HTTPAcceptor::_bind: Failed to set domain socket "
                     "permissions.");
@@ -497,13 +506,16 @@ void HTTPAcceptor::_bind()
 
     //int const _maxConnectionQueueLength = 15;
 
-    if (listen(_rep->socket, _maxConnectionQueueLength) < 0)
+    if (::listen(_rep->socket, _maxConnectionQueueLength) < 0)
     {
+        MessageLoaderParms parms(
+            "Common.HTTPAcceptor.FAILED_LISTEN_SOCKET",
+            "Failed to listen on socket {0}: {1}.",
+            (int)_rep->socket,PEGASUS_SYSTEM_NETWORK_ERRORMSG_NLS );
+        
         Socket::close(_rep->socket);
         delete _rep;
         _rep = 0;
-        MessageLoaderParms parms("Common.HTTPAcceptor.FAILED_BIND_SOCKET",
-            "Failed to bind socket");
         PEG_TRACE_CSTRING(TRC_DISCARDED_DATA, Tracer::LEVEL2,
             "HTTPAcceptor::_bind: Failed to bind socket(1).");
         throw BindFailedException(parms);
@@ -625,10 +637,13 @@ Uint32 HTTPAcceptor::getOutstandingRequestCount() const
     if (_rep)
     {
         AutoMutex autoMut(_rep->_connection_mut);
-        if (_rep->connections.size() > 0)
+        for (Uint32 i = 0, n = _rep->connections.size(); i < n; i++)
         {
-            HTTPConnection* connection = _rep->connections[0];
-            count = connection->getRequestCount();
+            HTTPConnection* connection = _rep->connections[i];
+            if (connection->isResponsePending())
+            {
+                count++;
+            }
         }
     }
     return count;
@@ -648,6 +663,11 @@ void HTTPAcceptor::setSocketWriteTimeout(Uint32 socketWriteTimeout)
     _socketWriteTimeout = socketWriteTimeout;
 }
 
+void HTTPAcceptor::setIdleConnectionTimeout(Uint32 idleConnectionTimeoutSeconds)
+{
+    _idleConnectionTimeoutSeconds = idleConnectionTimeoutSeconds;
+}
+
 void HTTPAcceptor::unbind()
 {
     if (_rep)
@@ -736,15 +756,17 @@ void HTTPAcceptor::_acceptConnection()
 #endif
     }
 
-    SocketHandle socket;
-#ifdef PEGASUS_OS_TYPE_WINDOWS
-    socket = accept(_rep->socket, accept_address, &address_size);
-#else
-    while (
-        ((socket = accept(_rep->socket, accept_address, &address_size)) == -1)
-        && (errno == EINTR))
-        ;
-#endif
+    // It is not necessary to handle EINTR errors from this accept() call.
+    // An EINTR error should not occur on a non-blocking socket.  If the
+    // listen socket is blocking and EINTR occurs, the new socket connection
+    // is not accepted here.
+
+    // EAGAIN errors are also not handled here.  An EAGAIN error should not
+    // occur after select() indicates that the listen socket is available for
+    // reading.  If the accept() fails with an EAGAIN error code, a new
+    // connection is not accepted here.
+
+    SocketHandle socket = accept(_rep->socket, accept_address, &address_size);
 
     if (socket == PEGASUS_SOCKET_ERROR)
     {
@@ -769,6 +791,30 @@ void HTTPAcceptor::_acceptConnection()
             "HTTPAcceptor: accept() failed");
         return;
     }
+#ifndef PEGASUS_OS_TYPE_WINDOWS
+    // We need to ensure that the socket number is not higher than
+    // what fits into FD_SETSIZE, because we else won't be able to select on it
+    // and won't ever communicate correct on that socket.
+    if (socket >= FD_SETSIZE)
+    {
+        // the remote connection is invalid, destroy client address.
+        delete accept_address;
+        
+        Logger::put(Logger::STANDARD_LOG, System::CIMSERVER, Logger::TRACE,
+            "HTTPAcceptor out of available sockets. "
+                "Closing connection to the new client.");
+
+        PEG_TRACE(
+            (TRC_DISCARDED_DATA,
+             Tracer::LEVEL2,
+             "accept() returned too large socket number %d.",
+             socket));
+        
+        // close the connection
+        Socket::close(socket);
+        return;
+    }
+#endif
 
     String ipAddress;
 
@@ -837,15 +883,12 @@ void HTTPAcceptor::_acceptConnection()
     SharedPtr<MP_Socket> mp_socket(new MP_Socket(
         socket, _sslcontext, _sslContextObjectLock, ipAddress));
 
+    mp_socket->disableBlocking();
     mp_socket->setSocketWriteTimeout(_socketWriteTimeout);
 
-    // Perform the SSL handshake, if applicable.  Make the socket non-blocking
-    // for this operation so we can send it back to the Monitor's select() loop
-    // if it takes a while.
+    // Perform the SSL handshake, if applicable.
 
-    mp_socket->disableBlocking();
     Sint32 socketAcceptStatus = mp_socket->accept();
-    mp_socket->enableBlocking();
 
     if (socketAcceptStatus < 0)
     {
@@ -860,11 +903,19 @@ void HTTPAcceptor::_acceptConnection()
     HTTPConnection* connection = new HTTPConnection(_monitor, mp_socket,
         ipAddress, this, static_cast<MessageQueue *>(_outputMessageQueue));
 
+    if (_idleConnectionTimeoutSeconds)
+    {
+        connection->_idleConnectionTimeoutSeconds = 
+            _idleConnectionTimeoutSeconds;
+        Time::gettimeofday(&connection->_idleStartTime);
+    }
+
     if (socketAcceptStatus == 0)
     {
         PEG_TRACE_CSTRING(TRC_HTTP, Tracer::LEVEL2,
             "HTTPAcceptor: SSL_accept() pending");
         connection->_acceptPending = true;
+        Time::gettimeofday(&connection->_acceptPendingStartTime);
     }
 
     // Solicit events on this new connection's socket:
