@@ -58,10 +58,6 @@ PEGASUS_USING_STD;
 
 PEGASUS_NAMESPACE_BEGIN
 
-// initialize the request count
-
-AtomicInt HTTPConnection::_requestCount(0);
-
 ////////////////////////////////////////////////////////////////////////////////
 //
 // Local routines:
@@ -206,11 +202,12 @@ HTTPConnection::HTTPConnection(
     _contentOffset(-1),
     _contentLength(-1),
     _connectionClosePending(false),
-    _acceptPending(false)
+    _acceptPending(false),
+    _firstRead(true),
+    _idleConnectionTimeoutSeconds(0)
 {
     PEG_METHOD_ENTER(TRC_HTTP, "HTTPConnection::HTTPConnection");
 
-    _socket->disableBlocking();
     _authInfo.reset(new AuthenticationInfo(true));
 
 #ifndef PEGASUS_PLATFORM_ZOS_ZSERIES_IBM
@@ -251,6 +248,16 @@ HTTPConnection::~HTTPConnection()
 {
     PEG_METHOD_ENTER(TRC_HTTP, "HTTPConnection::~HTTPConnection");
 
+    // We need to acquire this mutex in order to give handleEnqueue()
+    // a chance to finish processing. If we don't, we may run into a
+    // situation where the connection is marked to be closed by the
+    // idle connection timeout mechanism and there are no pending 
+    // responses (the _responsePending flag is cleared in 
+    // _handleWriteEvent()). This causes the monitor to clean up the
+    // connection. But if processing is not out of 
+    // HTTPConnection::handleEnqueue(), we are running a risk of 
+    // accessing a deleted object and crashing cimserver.
+    AutoMutex connectionLock(_connection_mut);
      _socket->close();
 
     PEG_METHOD_EXIT();
@@ -304,6 +311,53 @@ void HTTPConnection::handleEnqueue(Message *message)
     PEG_METHOD_EXIT();
 }
 
+/*
+ * Used on Server side to close outstanding connections waiting for SSL
+ * handshake to complete if timeout expired or to close idle connections if 
+ * idleConnectionTimeout config property value has specified.
+ * Returns 'true' if connection is closed (or is closePending).
+ * timeNow will be updated to current time if connection's _idleStartTime
+ * is greater than timeNow.
+*/
+
+Boolean HTTPConnection::closeConnectionOnTimeout(struct timeval* timeNow)
+{
+    // if SSL Handshake is not complete.
+    if (_acceptPending)
+    {
+        PEGASUS_ASSERT(!_isClient());
+        if ((timeNow->tv_sec - _acceptPendingStartTime.tv_sec >
+                 PEGASUS_SSL_ACCEPT_TIMEOUT_SECONDS) &&
+            (timeNow->tv_sec > _acceptPendingStartTime.tv_sec))
+        {
+            PEG_TRACE_CSTRING(TRC_DISCARDED_DATA, Tracer::LEVEL2,
+                "HTTPConnection: close acceptPending connection for timeout");
+            _closeConnection(); 
+            return true;  // return 'true' to indicate connection was closed
+        }
+    }
+    // else if connection timeout is active
+    else if (_idleConnectionTimeoutSeconds)
+    {
+        // For performance reasons timeNow is calculated only once in Monitor.
+        // Update timeNow if connection's _idleStartTime has more recent time.
+        if (timeNow->tv_sec < _idleStartTime.tv_sec)
+        {
+            Time::gettimeofday(timeNow);
+        }
+        else if ((Uint32)(timeNow->tv_sec - _idleStartTime.tv_sec) >
+            _idleConnectionTimeoutSeconds)
+        {
+            PEG_TRACE((TRC_DISCARDED_DATA, Tracer::LEVEL2,
+                "HTTPConnection: close idle connection for timeout "
+                "of %d seconds\n", _idleConnectionTimeoutSeconds));
+            _closeConnection();
+            return true;  // return 'true' to indicate connection was closed
+        }
+    }
+    return false;  // connection was not closed
+}
+
 /*
  * handle the message coming down from the above. This is shared client and
  * server code. If the message is coming in chunks, then validate the chunk
@@ -941,19 +995,17 @@ Boolean HTTPConnection::_handleWriteEvent(Message &message)
         // decrement request count
         //
 
-        _requestCount--;
         _responsePending = false;
 
         if (httpStatus.size() == 0)
         {
-            static const char msg[] =
-                "A response has been sent (%d of %d bytes have been written).\n"
-                "There are %d requests pending within the CIM Server.\n"
-                "A total of %d requests have been processed on this "
-                    "connection.";
-
-            PEG_TRACE((TRC_HTTP, Tracer::LEVEL4, msg, totalBytesWritten,
-                messageLength, _requestCount.get(), _connectionRequestCount));
+            PEG_TRACE((TRC_HTTP, Tracer::LEVEL4,
+                "A response has been sent (%d of %u bytes have been written). "
+                    "A total of %u requests have been processed on this "
+                    "connection.",
+                totalBytesWritten,
+                messageLength,
+                _connectionRequestCount));
         }
 
         //
@@ -974,6 +1026,11 @@ Boolean HTTPConnection::_handleWriteEvent(Message &message)
             }
             else
             {
+                // Update the connection idle time.
+                if (_idleConnectionTimeoutSeconds)
+                {
+                    Time::gettimeofday(&_idleStartTime);
+                }
                 PEG_TRACE((TRC_HTTP, Tracer::LEVEL2,
                     "Now setting state to %d", _MonitorEntry::IDLE));
                 _monitor->setState (_entry_index, _MonitorEntry::IDLE);
@@ -1824,7 +1881,6 @@ void HTTPConnection::_handleReadEventFailure(
         httpStatus + httpDetailDelimiter + httpDetail +
             httpDetailDelimiter + cimError);
 
-    _requestCount++;
     Buffer message;
     message = XmlWriter::formatHttpErrorRspMessage(httpStatus, cimError,
         httpDetail);
@@ -1926,6 +1982,34 @@ void HTTPConnection::_handleReadEvent()
 
         Sint32 n = _socket->read(buffer, sizeof(buffer)-1);
 
+        // Check if this was the first read of a connection to the server.
+        // This has to happen inside the read loop, because there can be
+        // an incomplete SSL read.
+        if (_firstRead && n > 5 && !_isClient())
+        {
+            // The first bytes of a connection to the server have to contain
+            // a valid cim-over-http HTTP Method (M-POST or POST).
+            if ((strncmp(buffer, "POST", 4) != 0) &&
+                (strncmp(buffer, "M-POST", 6) != 0))
+            {
+                _clearIncoming();
+
+                PEG_TRACE((TRC_HTTP, Tracer::LEVEL4,
+                      "This Request has non-valid CIM-HTTP Method: "
+                      "%02X %02X %02X %02X %02X %02X",
+                      buffer[0],buffer[1],buffer[2],
+                      buffer[3],buffer[4],buffer[5]));
+
+                // Try to send message to client. 
+                // This function also closes the connection.
+                _handleReadEventFailure(HTTP_STATUS_NOTIMPLEMENTED);
+
+                PEG_METHOD_EXIT();
+                return;
+            }
+            _firstRead = false;
+        }
+
         if (n <= 0)
         {
             // It is possible that SSL_read was not able to
@@ -2012,15 +2096,20 @@ void HTTPConnection::_handleReadEvent()
                 getQueueId()));
             _closeConnection();
 
-            PEG_TRACE((TRC_HTTP, Tracer::LEVEL4,
-                "_requestCount = %d", _requestCount.get()));
-
-            PEG_METHOD_EXIT();
-            return;
+            //
+            // If we are executing on the server side, the connection
+            // is closed, return. Do not forward an empty HTTP message.
+            //
+            if (!_isClient())
+            {
+                PEG_METHOD_EXIT();
+                return;
+            }
         }
 
-        // A message was received, so process it.  If the connection was
-        // closed, we will handle that in the next iteration.
+        // If the connection was closed and we are executing on the client
+        // side send an empty HTTP message. Otherwise, a message was
+        // received, so process it.
 
         HTTPMessage* message = new HTTPMessage(_incomingBuffer, getQueueId());
         message->authInfo = _authInfo.get();
@@ -2028,6 +2117,18 @@ void HTTPConnection::_handleReadEvent()
         message->contentLanguages = contentLanguages;
         message->dest = _outputMessageQueue->getQueueId();
 
+        //
+        // The _closeConnection method sets the _connectionClosePending flag.
+        // If we are executing on the client side and the
+        // _connectionClosePending flag is set, send an empty HTTP message.
+        //
+        if (_connectionClosePending)
+        {
+            _outputMessageQueue->enqueue(message);
+            PEG_METHOD_EXIT();
+            return;
+        }
+
         if (_isClient() == false)
         {
             PEG_TRACE((
@@ -2042,17 +2143,13 @@ void HTTPConnection::_handleReadEvent()
         //
         // increment request count
         //
-        _requestCount++;
         _connectionRequestCount++;
         _responsePending = true;
 
-        PEG_TRACE((TRC_HTTP, Tracer::LEVEL4,
-            "_requestCount = %d", _requestCount.get()));
-
         //
         // Set the entry status to BUSY.
         //
-        if (_isClient() == false && !_connectionClosePending)
+        if (_isClient() == false)
         {
             PEG_TRACE((TRC_HTTP, Tracer::LEVEL2,
                 "Now setting state to %d", _MonitorEntry::BUSY));
@@ -2076,9 +2173,9 @@ void HTTPConnection::_handleReadEvent()
     PEG_METHOD_EXIT();
 }
 
-Uint32 HTTPConnection::getRequestCount()
+Boolean HTTPConnection::isResponsePending()
 {
-    return _requestCount.get();
+    return _responsePending;
 }
 
 Boolean HTTPConnection::run(Uint32 milliseconds)
