@@ -42,13 +42,302 @@
 #include <Pegasus/Common/MessageQueueService.h>
 #include <Pegasus/Common/Exception.h>
 #include "ArrayIterator.h"
+#include "HostAddress.h"
 #include <errno.h>
 
 PEGASUS_USING_STD;
 
 PEGASUS_NAMESPACE_BEGIN
 
-static AtomicInt _connections(0);
+////////////////////////////////////////////////////////////////////////////////
+//
+// Tickler
+//
+////////////////////////////////////////////////////////////////////////////////
+
+Tickler::Tickler()
+    : _listenSocket(PEGASUS_INVALID_SOCKET),
+      _clientSocket(PEGASUS_INVALID_SOCKET),
+      _serverSocket(PEGASUS_INVALID_SOCKET)
+{
+    try
+    {
+        _initialize();
+    }
+    catch (...)
+    {
+        _uninitialize();
+        throw;
+    }
+}
+
+Tickler::~Tickler()
+{
+    _uninitialize();
+}
+
+void Tickler::notify()
+{
+    Socket::write(_clientSocket, "\0", 1);
+}
+
+void Tickler::reset()
+{
+    // Clear all bytes from the tickle socket
+    char buffer[32];
+    while (Socket::read(_serverSocket, buffer, 32) > 0)
+    {
+    }
+}
+
+#if defined(PEGASUS_OS_TYPE_UNIX)
+
+// Use an anonymous pipe for the tickle connection.
+
+void Tickler::_initialize()
+{
+    int fds[2];
+
+    if (pipe(fds) == -1)
+    {
+        MessageLoaderParms parms(
+            "Common.Monitor.TICKLE_CREATE",
+            "Received error number $0 while creating the internal socket.",
+            getSocketError());
+        throw Exception(parms);
+    }
+
+    _serverSocket = fds[0];
+    _clientSocket = fds[1];
+
+    Socket::disableBlocking(_serverSocket);
+}
+
+#else
+
+// Use an external loopback socket connection to allow the tickle socket to
+// be included in the select() array on non-Unix platforms.
+
+void Tickler::_initialize()
+{
+    //
+    // Set up the addresses for the listen, client, and server sockets
+    // based on whether IPv6 is enabled.
+    //
+
+    Socket::initializeInterface();
+
+# ifdef PEGASUS_ENABLE_IPV6
+    struct sockaddr_storage listenAddress;
+    struct sockaddr_storage clientAddress;
+    struct sockaddr_storage serverAddress;
+# else
+    struct sockaddr_in listenAddress;
+    struct sockaddr_in clientAddress;
+    struct sockaddr_in serverAddress;
+# endif
+
+    int addressFamily;
+    SocketLength addressLength;
+
+    memset(&listenAddress, 0, sizeof (listenAddress));
+
+# ifdef PEGASUS_ENABLE_IPV6
+    if (System::isIPv6StackActive())
+    {
+        // Use the IPv6 loopback address for the listen sockets
+        HostAddress::convertTextToBinary(
+            HostAddress::AT_IPV6,
+            "::1",
+            &reinterpret_cast<struct sockaddr_in6*>(&listenAddress)->sin6_addr);
+        listenAddress.ss_family = AF_INET6;
+        reinterpret_cast<struct sockaddr_in6*>(&listenAddress)->sin6_port = 0;
+
+        addressFamily = AF_INET6;
+        addressLength = sizeof(struct sockaddr_in6);
+    }
+    else
+# endif
+    {
+        // Use the IPv4 loopback address for the listen sockets
+        HostAddress::convertTextToBinary(
+            HostAddress::AT_IPV4,
+            "127.0.0.1",
+            &reinterpret_cast<struct sockaddr_in*>(
+                &listenAddress)->sin_addr.s_addr);
+        reinterpret_cast<struct sockaddr_in*>(&listenAddress)->sin_family =
+            AF_INET;
+        reinterpret_cast<struct sockaddr_in*>(&listenAddress)->sin_port = 0;
+
+        addressFamily = AF_INET;
+        addressLength = sizeof(struct sockaddr_in);
+    }
+
+    // Use the same address for the client socket as the listen socket
+    clientAddress = listenAddress;
+
+    //
+    // Set up a listen socket to allow the tickle client and server to connect
+    //
+
+    // Create the listen socket
+    if ((_listenSocket = Socket::createSocket(addressFamily, SOCK_STREAM, 0)) ==
+             PEGASUS_INVALID_SOCKET)
+    {
+        MessageLoaderParms parms(
+            "Common.Monitor.TICKLE_CREATE",
+            "Received error number $0 while creating the internal socket.",
+            getSocketError());
+        throw Exception(parms);
+    }
+
+    // Bind the listen socket to the loopback address
+    if (::bind(
+            _listenSocket,
+            reinterpret_cast<struct sockaddr*>(&listenAddress),
+            addressLength) < 0)
+    {
+        MessageLoaderParms parms(
+            "Common.Monitor.TICKLE_BIND",
+            "Received error number $0 while binding the internal socket.",
+            getSocketError());
+        throw Exception(parms);
+    }
+
+    // Listen for a connection from the tickle client
+    if ((::listen(_listenSocket, 3)) < 0)
+    {
+        MessageLoaderParms parms(
+            "Common.Monitor.TICKLE_LISTEN",
+            "Received error number $0 while listening to the internal socket.",
+            getSocketError());
+        throw Exception(parms);
+    }
+
+    // Verify we have the correct listen socket
+    SocketLength tmpAddressLength = addressLength;
+    int sock = ::getsockname(
+        _listenSocket,
+        reinterpret_cast<struct sockaddr*>(&listenAddress),
+        &tmpAddressLength);
+    if (sock < 0)
+    {
+        MessageLoaderParms parms(
+            "Common.Monitor.TICKLE_SOCKNAME",
+            "Received error number $0 while getting the internal socket name.",
+            getSocketError());
+        throw Exception(parms);
+    }
+
+    //
+    // Set up the client side of the tickle connection.
+    //
+
+    // Create the client socket
+    if ((_clientSocket = Socket::createSocket(addressFamily, SOCK_STREAM, 0)) ==
+             PEGASUS_INVALID_SOCKET)
+    {
+        MessageLoaderParms parms(
+            "Common.Monitor.TICKLE_CLIENT_CREATE",
+            "Received error number $0 while creating the internal client "
+                "socket.",
+            getSocketError());
+        throw Exception(parms);
+    }
+
+    // Bind the client socket to the loopback address
+    if (::bind(
+            _clientSocket,
+            reinterpret_cast<struct sockaddr*>(&clientAddress),
+            addressLength) < 0)
+    {
+        MessageLoaderParms parms(
+            "Common.Monitor.TICKLE_CLIENT_BIND",
+            "Received error number $0 while binding the internal client "
+                "socket.",
+            getSocketError());
+        throw Exception(parms);
+    }
+
+    // Connect the client socket to the listen socket address
+    if (::connect(
+            _clientSocket,
+            reinterpret_cast<struct sockaddr*>(&listenAddress),
+            addressLength) < 0)
+    {
+        MessageLoaderParms parms(
+            "Common.Monitor.TICKLE_CLIENT_CONNECT",
+            "Received error number $0 while connecting the internal client "
+                "socket.",
+            getSocketError());
+        throw Exception(parms);
+    }
+
+    //
+    // Set up the server side of the tickle connection.
+    //
+
+    tmpAddressLength = addressLength;
+
+    // Accept the client socket connection.
+    _serverSocket = ::accept(
+        _listenSocket,
+        reinterpret_cast<struct sockaddr*>(&serverAddress),
+        &tmpAddressLength);
+
+    if (_serverSocket == PEGASUS_SOCKET_ERROR)
+    {
+        MessageLoaderParms parms(
+            "Common.Monitor.TICKLE_ACCEPT",
+            "Received error number $0 while accepting the internal socket "
+                "connection.",
+            getSocketError());
+        throw Exception(parms);
+    }
+
+    //
+    // Close the listen socket and make the other sockets non-blocking
+    //
+
+    Socket::close(_listenSocket);
+    _listenSocket = PEGASUS_INVALID_SOCKET;
+
+    Socket::disableBlocking(_serverSocket);
+    Socket::disableBlocking(_clientSocket);
+}
+
+#endif
+
+void Tickler::_uninitialize()
+{
+    PEG_TRACE_CSTRING(TRC_HTTP, Tracer::LEVEL4, "uninitializing interface");
+
+    try
+    {
+        if (_serverSocket != PEGASUS_INVALID_SOCKET)
+        {
+            Socket::close(_serverSocket);
+            _serverSocket = PEGASUS_INVALID_SOCKET;
+        }
+        if (_clientSocket != PEGASUS_INVALID_SOCKET)
+        {
+            Socket::close(_clientSocket);
+            _clientSocket = PEGASUS_INVALID_SOCKET;
+        }
+        if (_listenSocket != PEGASUS_INVALID_SOCKET)
+        {
+            Socket::close(_listenSocket);
+            _listenSocket = PEGASUS_INVALID_SOCKET;
+        }
+    }
+    catch (...)
+    {
+        PEG_TRACE_CSTRING(TRC_HTTP, Tracer::LEVEL4,
+            "Failed to close tickle sockets");
+    }
+    Socket::uninitializeInterface();
+}
+
 
 ////////////////////////////////////////////////////////////////////////////////
 //
@@ -60,21 +349,18 @@ static AtomicInt _connections(0);
 Monitor::Monitor()
    : _stopConnections(0),
      _stopConnectionsSem(0),
-     _solicitSocketCount(0),
-     _tickle_client_socket(-1),
-     _tickle_server_socket(-1),
-     _tickle_peer_socket(-1)
+     _solicitSocketCount(0)
 {
     int numberOfMonitorEntriesToAllocate = MAX_NUMBER_OF_MONITOR_ENTRIES;
-    Socket::initializeInterface();
     _entries.reserveCapacity(numberOfMonitorEntriesToAllocate);
 
-    // setup the tickler
-    initializeTickler();
+    // Create a MonitorEntry for the Tickler and set its state to IDLE so the
+    // Monitor will watch for its events.
+    _MonitorEntry entry(_tickler.getReadHandle(), 1, INTERNAL);
+    entry._status = _MonitorEntry::IDLE;
+    _entries.append(entry);
 
-    // Start the count at 1 because initilizeTickler()
-    // has added an entry in the first position of the
-    // _entries array
+    // Start the count at 1 because _entries[0] is the Tickler
     for (int i = 1; i < numberOfMonitorEntriesToAllocate; i++)
     {
        _MonitorEntry entry(0, 0, 0);
@@ -84,248 +370,13 @@ Monitor::Monitor()
 
 Monitor::~Monitor()
 {
-    uninitializeTickler();
-    Socket::uninitializeInterface();
     PEG_TRACE_CSTRING(TRC_HTTP, Tracer::LEVEL4,
                   "returning from monitor destructor");
 }
-void Monitor::uninitializeTickler()
-{
-    PEG_TRACE_CSTRING(TRC_HTTP, Tracer::LEVEL4, "uninitializing interface");
-
-    try
-    {
-        if (_tickle_peer_socket >= 0)
-        {
-            Socket::close(_tickle_peer_socket);
-        }
-        if (_tickle_client_socket >= 0)
-        {
-            Socket::close(_tickle_client_socket);
-        }
-        if (_tickle_server_socket >= 0)
-        {
-            Socket::close(_tickle_server_socket);
-        }
-    }
-    catch (...)
-    {
-        PEG_TRACE_CSTRING(TRC_HTTP, Tracer::LEVEL4,
-                  "Failed to close tickle sockets");
-    }
-
-}
-
-void Monitor::initializeTickler()
-{
-    /*
-       NOTE: On any errors trying to
-             setup out tickle connection,
-             throw an exception/end the server
-    */
-
-    /* setup the tickle server/listener */
-    // try until the tcpip is restarted
-    do
-    {
-        // get a socket for the server side
-        if ((_tickle_server_socket =
-                 Socket::createSocket(PF_INET, SOCK_STREAM, 0)) ==
-             PEGASUS_INVALID_SOCKET)
-        {
-            MessageLoaderParms parms(
-                "Common.Monitor.TICKLE_CREATE",
-                "Received error number $0 while creating the internal socket.",
-                getSocketError());
-            throw Exception(parms);
-        }
-
-        // initialize the address
-        memset(&_tickle_server_addr, 0, sizeof(_tickle_server_addr));
-
-        _tickle_server_addr.sin_addr.s_addr = inet_addr("127.0.0.1");
-        _tickle_server_addr.sin_family = PF_INET;
-        _tickle_server_addr.sin_port = 0;
-
-        SocketLength _addr_size = sizeof(_tickle_server_addr);
-
-        // bind server side to socket
-        if ((::bind(_tickle_server_socket,
-                 reinterpret_cast<struct sockaddr*>(&_tickle_server_addr),
-                 sizeof(_tickle_server_addr))) < 0)
-        {
-#ifdef PEGASUS_OS_ZOS
-            MessageLoaderParms parms(
-                "Common.Monitor.TICKLE_BIND_LONG",
-                "Received error:$0 while binding the internal socket.",
-                strerror(errno));
-#else
-            MessageLoaderParms parms(
-                "Common.Monitor.TICKLE_BIND",
-                "Received error number $0 while binding the internal socket.",
-                getSocketError());
-#endif
-            throw Exception(parms);
-        }
-
-        // tell the kernel we are a server
-        if ((::listen(_tickle_server_socket, 3)) < 0)
-        {
-            MessageLoaderParms parms(
-                "Common.Monitor.TICKLE_LISTEN",
-                "Received error number $0 while listening to the internal "
-                    "socket.",
-                getSocketError());
-            throw Exception(parms);
-        }
-
-        // make sure we have the correct socket for our server
-        int sock = ::getsockname(
-            _tickle_server_socket,
-            reinterpret_cast<struct sockaddr*>(&_tickle_server_addr),
-            &_addr_size);
-        if (sock < 0)
-        {
-            MessageLoaderParms parms(
-                "Common.Monitor.TICKLE_SOCKNAME",
-                "Received error number $0 while getting the internal socket "
-                    "name.",
-                getSocketError());
-            throw Exception(parms);
-        }
-
-        /* set up the tickle client/connector */
-
-        // get a socket for our tickle client
-        if ((_tickle_client_socket =
-                 Socket::createSocket(PF_INET, SOCK_STREAM, 0)) ==
-            PEGASUS_INVALID_SOCKET)
-        {
-            MessageLoaderParms parms(
-                "Common.Monitor.TICKLE_CLIENT_CREATE",
-                "Received error number $0 while creating the internal client "
-                    "socket.",
-                getSocketError());
-            throw Exception(parms);
-        }
-
-        // setup the address of the client
-        memset(&_tickle_client_addr, 0, sizeof(_tickle_client_addr));
-
-        _tickle_client_addr.sin_addr.s_addr = inet_addr("127.0.0.1");
-        _tickle_client_addr.sin_family = PF_INET;
-        _tickle_client_addr.sin_port = 0;
-
-        // bind socket to client side
-        if ((::bind(_tickle_client_socket,
-                 reinterpret_cast<struct sockaddr*>(&_tickle_client_addr),
-                 sizeof(_tickle_client_addr))) < 0)
-        {
-            MessageLoaderParms parms(
-                "Common.Monitor.TICKLE_CLIENT_BIND",
-                "Received error number $0 while binding the internal client "
-                    "socket.",
-                getSocketError());
-            throw Exception(parms);
-        }
-
-        // connect to server side
-        if ((::connect(_tickle_client_socket,
-                 reinterpret_cast<struct sockaddr*>(&_tickle_server_addr),
-                 sizeof(_tickle_server_addr))) < 0)
-        {
-            MessageLoaderParms parms(
-                "Common.Monitor.TICKLE_CLIENT_CONNECT",
-                "Received error number $0 while connecting the internal "
-                    "client socket.",
-                getSocketError());
-            throw Exception(parms);
-        }
-
-        /* set up the slave connection */
-        memset(&_tickle_peer_addr, 0, sizeof(_tickle_peer_addr));
-        SocketLength peer_size = sizeof(_tickle_peer_addr);
-        Threads::sleep(1);
-
-        // this call may fail, we will try a max of 20 times to establish
-        // this peer connection
-        if ((_tickle_peer_socket = ::accept(_tickle_server_socket,
-                 reinterpret_cast<struct sockaddr*>(&_tickle_peer_addr),
-                 &peer_size)) < 0)
-        {
-            if (_tickle_peer_socket == PEGASUS_SOCKET_ERROR &&
-                getSocketError() == PEGASUS_NETWORK_TRYAGAIN)
-            {
-                int retries = 0;
-                do
-                {
-                    Threads::sleep(1);
-                    _tickle_peer_socket = ::accept(
-                        _tickle_server_socket,
-                        reinterpret_cast<struct sockaddr*>(&_tickle_peer_addr),
-                        &peer_size);
-                    retries++;
-                } while (_tickle_peer_socket == PEGASUS_SOCKET_ERROR &&
-                         getSocketError() == PEGASUS_NETWORK_TRYAGAIN &&
-                         retries < 20);
-            }
-            // TCP/IP is down, destroy sockets and retry again.
-            if (_tickle_peer_socket == PEGASUS_SOCKET_ERROR &&
-                getSocketError() == PEGASUS_NETWORK_TCPIP_STOPPED)
-            {
-                // destroy everything
-                uninitializeTickler();
-                // retry again.
-                continue;
-            }
-        }
-        if (_tickle_peer_socket == PEGASUS_SOCKET_ERROR)
-        {
-            MessageLoaderParms parms(
-                "Common.Monitor.TICKLE_ACCEPT",
-                "Received error number $0 while accepting the internal "
-                    "socket connection.",
-                getSocketError());
-            throw Exception(parms);
-        }
-        else
-        {
-            // socket is ok
-            break;
-        }
-    } while (1); // try until TCP/IP is restarted
-
-    Socket::disableBlocking(_tickle_peer_socket);
-    Socket::disableBlocking(_tickle_client_socket);
-
-    // add the tickler to the list of entries to be monitored and set to
-    // IDLE because Monitor only
-    // checks entries with IDLE state for events
-    _MonitorEntry entry(_tickle_peer_socket, 1, INTERNAL);
-    entry._status = _MonitorEntry::IDLE;
-
-    // is the tickler initalized as first socket on startup ?
-    if (_entries.size()==0)
-    {
-       // if yes, append a new entry
-       _entries.append(entry);
-    }
-    else
-    {
-       // if not, overwrite the tickler entry with new socket
-       _entries[0]=entry;
-    }
-}
 
 void Monitor::tickle()
 {
-    static char _buffer[] =
-    {
-      '0','0'
-    };
-
-    AutoMutex autoMutex(_tickle_mutex);
-    Socket::write(_tickle_client_socket,&_buffer, 2);
+    _tickler.notify();
 }
 
 void Monitor::setState(
@@ -339,8 +390,6 @@ void Monitor::setState(
 
 void Monitor::run(Uint32 milliseconds)
 {
-
-
     struct timeval tv = {milliseconds/1000, milliseconds%1000*1000};
 
     fd_set fdread;
@@ -476,6 +525,9 @@ void Monitor::run(Uint32 milliseconds)
 #endif
     _entry_mut.lock();
 
+    struct timeval timeNow;
+    Time::gettimeofday(&timeNow);
+
     // After enqueue a message and the autoEntryMutex has been released and
     // locked again, the array of _entries can be changed. The ArrayIterator
     // has be reset with the original _entries
@@ -519,34 +571,29 @@ void Monitor::run(Uint32 milliseconds)
                             "entries[indx].type for indx = %d is "
                                 "Monitor::CONNECTION",
                             indx));
-                        static_cast<HTTPConnection *>(q)->_entry_index = indx;
-
-                        // Do not update the entry just yet. The entry gets
-                        // updated once the request has been read.
-                        //entries[indx]._status = _MonitorEntry::BUSY;
-
-                        // If allocate_and_awaken failure, retry on next
-                        // iteration
-/* Removed for PEP 183.
-                        if (!MessageQueueService::get_thread_pool()->
-                                allocate_and_awaken((void *)q, _dispatch))
-                        {
-                            PEG_TRACE_CSTRING(TRC_DISCARDED_DATA,
-                                Tracer::LEVEL2,
-                                "Monitor::run: Insufficient resources to "
-                                    "process request.");
-                            entries[indx]._status = _MonitorEntry::IDLE;
-                            return true;
-                        }
-*/
-// Added for PEP 183
                         HTTPConnection *dst =
                             reinterpret_cast<HTTPConnection *>(q);
+                        dst->_entry_index = indx;
+
+                        // Update idle start time because we have received some
+                        // data. Any data is good data at this point, and we'll
+                        // keep the connection alive, even if we've exceeded
+                        // the idleConnectionTimeout, which will be checked
+                        // when we call closeConnectionOnTimeout() next.
+                        Time::gettimeofday(&dst->_idleStartTime);
+
+                        // Check for accept pending (ie. SSL handshake pending)
+                        // or idle connection timeouts for sockets from which
+                        // we received data (avoiding extra queue lookup below).
+                        if (!dst->closeConnectionOnTimeout(&timeNow))
+                        {
+
                         PEG_TRACE((TRC_HTTP, Tracer::LEVEL4,
                             "Monitor::_dispatch: entering run() for "
                                 "indx = %d, queueId = %d, q = %p",
                             dst->_entry_index,
-                            dst->_monitor->_entries[dst->_entry_index].queueId,
+                                dst->_monitor->
+                                    _entries[dst->_entry_index].queueId,
                             dst));
 
                         try
@@ -561,58 +608,11 @@ void Monitor::run(Uint32 milliseconds)
                         PEG_TRACE((TRC_HTTP, Tracer::LEVEL4,
                             "Monitor::_dispatch: exited run() for index %d",
                             dst->_entry_index));
-
-                        // It is possible the entry status may not be set to
-                        // busy.  The following will fail in that case.
-                        // PEGASUS_ASSERT(dst->_monitor->_entries[
-                        //     dst->_entry_index]._status.get() ==
-                        //    _MonitorEntry::BUSY);
-                        // Once the HTTPConnection thread has set the status
-                        // value to either Monitor::DYING or Monitor::IDLE,
-                        // it has returned control of the connection to the
-                        // Monitor.  It is no longer permissible to access
-                        // the connection or the entry in the _entries table.
-
-                        // The following is not relevant as the worker thread
-                        // or the reader thread will update the status of the
-                        // entry.
-                        //if (dst->_connectionClosePending)
-                        //{
-                        //  dst->_monitor->_entries[dst->_entry_index]._status =
-                        //    _MonitorEntry::DYING;
-                        //}
-                        //else
-                        //{
-                        //  dst->_monitor->_entries[dst->_entry_index]._status =
-                        //    _MonitorEntry::IDLE;
-                        //}
-// end Added for PEP 183
+                        }
                     }
                     else if (entries[indx]._type == Monitor::INTERNAL)
                     {
-                        // set ourself to BUSY,
-                        // read the data
-                        // and set ourself back to IDLE
-
-                        entries[indx]._status = _MonitorEntry::BUSY;
-                        static char buffer[2];
-                        Sint32 amt =
-                            Socket::read(entries[indx].socket,&buffer, 2);
-
-                        if (amt == PEGASUS_SOCKET_ERROR &&
-                            getSocketError() == PEGASUS_NETWORK_TCPIP_STOPPED)
-                        {
-                            PEG_TRACE_CSTRING(TRC_HTTP, Tracer::LEVEL4,
-                                "Monitor::run: Tickler socket got an IO error. "
-                                    "Going to re-create Socket and wait for "
-                                    "TCP/IP restart.");
-                            uninitializeTickler();
-                            initializeTickler();
-                        }
-                        else
-                        {
-                            entries[indx]._status = _MonitorEntry::IDLE;
-                        }
+                        _tickler.reset();
                     }
                     else
                     {
@@ -641,6 +641,33 @@ void Monitor::run(Uint32 milliseconds)
                 {
                 }
             }
+            // else check for accept pending (ie. SSL handshake pending) or
+            // idle connection timeouts for sockets from which we did not
+            // receive data.
+            else if ((entries[indx]._status.get() == _MonitorEntry::IDLE) &&
+                entries[indx]._type == Monitor::CONNECTION)
+            {
+                MessageQueue *q = MessageQueue::lookup(entries[indx].queueId);
+                HTTPConnection *dst = reinterpret_cast<HTTPConnection *>(q);
+                dst->_entry_index = indx;
+                dst->closeConnectionOnTimeout(&timeNow);
+            }
+        }
+    }
+    // else if "events" is zero (ie. select timed out) then we still need
+    // to check if there are any pending SSL handshakes that have timed out.
+    else
+    {
+        for (int indx = 0; indx < (int)entries.size(); indx++)
+        {
+            if ((entries[indx]._status.get() == _MonitorEntry::IDLE) &&
+                entries[indx]._type == Monitor::CONNECTION)
+            {
+                MessageQueue *q = MessageQueue::lookup(entries[indx].queueId);
+                HTTPConnection *dst = reinterpret_cast<HTTPConnection *>(q);
+                dst->_entry_index = indx;
+                dst->closeConnectionOnTimeout(&timeNow);
+            }
         }
     }
 }
