@@ -36,6 +36,13 @@
 
 PEGASUS_NAMESPACE_BEGIN
 
+Boolean ObjectNormalizer::_enableNormalization = false;
+
+void ObjectNormalizer::setEnableNormalization(Boolean value)
+{
+    _enableNormalization = value;
+}
+
 CIMQualifier _processQualifier(
     CIMConstQualifier& referenceQualifier,
     CIMConstQualifier& cimQualifier)
@@ -79,7 +86,7 @@ CIMQualifier _processQualifier(
     return normalizedQualifier;
 }
 
-CIMProperty ObjectNormalizer::_processProperty(
+CIMProperty ObjectNormalizer::processProperty(
     CIMConstProperty& referenceProperty,
     CIMConstProperty& instProperty,
     Boolean includeQualifiers,
@@ -374,7 +381,7 @@ CIMObjectPath ObjectNormalizer::processClassObjectPath(
     const CIMObjectPath& cimObjectPath) const
 {
     // pre-check
-    if (_cimClass.isUninitialized())
+    if (!_enableNormalization || _cimClass.isUninitialized())
     {
         // do nothing
         return cimObjectPath;
@@ -431,7 +438,7 @@ CIMObjectPath ObjectNormalizer::processInstanceObjectPath(
     const CIMObjectPath& cimObjectPath) const
 {
     // pre-check
-    if (_cimClass.isUninitialized())
+    if (!_enableNormalization || _cimClass.isUninitialized())
     {
         // do nothing
         return cimObjectPath;
@@ -534,7 +541,7 @@ CIMInstance ObjectNormalizer::processInstance(
     const CIMInstance& cimInstance) const
 {
     // pre-checks
-    if (_cimClass.isUninitialized())
+    if (!_enableNormalization || _cimClass.isUninitialized())
     {
         // do nothing
         return cimInstance;
@@ -588,20 +595,20 @@ CIMInstance ObjectNormalizer::processInstance(
 
     // check property names and types. any properties in the class but not
     // in the instance are implicitly dropped.
-    for (Uint32 i = 0, n = _cimClass.getPropertyCount(); i < n; i++)
+    for (Uint32 i = 0, n = cimInstance.getPropertyCount(); i < n; i++)
     {
-        CIMConstProperty referenceProperty = _cimClass.getProperty(i);
+        CIMConstProperty instProperty = cimInstance.getProperty(i);
 
-        Uint32 pos = cimInstance.findProperty(referenceProperty.getName());
+        Uint32 pos = _cimClass.findProperty(instProperty.getName());
 
         if (pos != PEG_NOT_FOUND)
         {
-            CIMConstProperty cimProperty = cimInstance.getProperty(pos);
+            CIMConstProperty cimProperty = _cimClass.getProperty(pos);
 
             CIMProperty normalizedProperty =
-                _processProperty(
-                    referenceProperty,
+                processProperty(
                     cimProperty,
+                    instProperty,
                     _includeQualifiers,
                     _includeClassOrigin,
                     _context.get(),
