@@ -53,6 +53,7 @@
 
 #if defined(PEGASUS_OS_SOLARIS)
 # include <string.h>
+# include <sys/sockio.h>
 #endif
 
 #if !defined(PEGASUS_OS_VMS) && \
@@ -73,6 +74,11 @@
 #include <time.h>
 #include <sys/time.h>
 #include "System.h"
+
+#ifdef PEGASUS_OS_PASE
+#include <ILEWrapper/ILEUtilities.h>
+#endif
+
 #include <Pegasus/Common/Tracer.h>
 #include <Pegasus/Common/InternalException.h>
 #include <Pegasus/Common/Mutex.h>
@@ -571,27 +577,22 @@ String System::getEffectiveUserName()
 String System::encryptPassword(const char* password, const char* salt)
 {
 #if defined(PEGASUS_OS_VMS)
-
     const size_t MAX_PASS_LEN = 1024;
     char pbBuffer[MAX_PASS_LEN] = {0};
-    int dwByteCount;
+    Uint32 dwByteCount;
     char pcSalt[3] = {0};
 
     strncpy(pcSalt, salt, 2);
     dwByteCount = strlen(password);
     memcpy(pbBuffer, password, dwByteCount);
 
-    for (int i=0; (i<dwByteCount) || (i>=MAX_PASS_LEN); i++)
+    for (Uint32 i=0; (i<dwByteCount) || (i>=MAX_PASS_LEN); i++)
     {
         (i%2 == 0) ? pbBuffer[i] ^= pcSalt[1] : pbBuffer[i] ^= pcSalt[0];
     }
-
     return String(pcSalt) + String((char *)pbBuffer);
-
 #else
-
-    return String(password);
-
+    return String(crypt(password, salt));
 #endif
 }
 
@@ -1048,7 +1049,8 @@ void System::syslog(const String& ident, Uint32 severity, const char* message)
 
     // reserve memory for the message string, also prepend
     // z/OS message id CFZ* if necessary
-    if (strncmp(message, "CFZ", 3) != 0)
+    if ((strncmp(message, "CFZ", 3) != 0) &&
+        (strncmp(message, "CEZ", 3) != 0) )
     {
         // reserve message + 11 char message prepend + 1 byte for \0 char
         zosMessageString = (char*) malloc(messageLength+ZOS_MSGID_LENGTH+1);
@@ -1155,6 +1157,53 @@ Boolean System::isIpOnNetworkInterface(Uint32 inIP)
     return false;
 }
 
+String System::getErrorMSG_NLS(int errorCode,int errorCode2)
+{
+#if defined(PEGASUS_OS_ZOS)
+
+    char buf[10];
+
+    sprintf(buf,"%08X",errorCode2);
+
+    MessageLoaderParms parms(
+           "Common.System.ERROR_MESSAGE.PEGASUS_OS_ZOS",
+           "$0 (error code $1, reason code 0x$2)",
+           strerror(errorCode),errorCode,buf);
+
+#else
+    MessageLoaderParms parms(
+           "Common.System.ERROR_MESSAGE.STANDARD",
+           "$0 (error code $1)",strerror(errorCode),errorCode);
+
+#endif
+
+     return MessageLoader::getMessage(parms);
+     
+}
+
+String System::getErrorMSG(int errorCode,int errorCode2)
+{
+
+    String buffer;
+
+    char strErrorCode[32];
+    sprintf(strErrorCode, "%d", errorCode);
+
+    buffer.append(strerror(errorCode));
+    buffer.append(" (error code ");
+    buffer.append(strErrorCode);
+
+#if defined(PEGASUS_OS_ZOS)
+    char strErrorCode2[10];
+    sprintf(strErrorCode2,"%08X",errorCode2);
+    buffer.append(", reason code 0x");
+    buffer.append(strErrorCode2);
+#endif
+
+    buffer.append(")");
+
+    return buffer;
+}
 
 ///////////////////////////////////////////////////////////////////////////////
 // AutoFileLock class
@@ -1250,4 +1299,69 @@ static SystemInitializer initializer;
 
 #endif /* PEGASUS_OS_AIX */
 
+Array<String> System::getInterfaceAddrs()
+{
+    Array<String> ips;
+
+#ifdef PEGASUS_ENABLE_IPV6
+
+#if defined(PEGASUS_HAS_GETIFADDRS)
+    struct ifaddrs *array, *addrs;
+    char buff[PEGASUS_INET6_ADDRSTR_LEN];
+
+    if (0 > getifaddrs(&array))
+    {
+        return ips;
+    }
+
+    for( addrs = array; addrs != NULL; addrs = addrs->ifa_next)
+    {
+        switch(addrs->ifa_addr->sa_family)
+        {
+            case AF_INET :
+                // Don't gather loop back addrs.
+                if (System::isLoopBack(AF_INET, 
+                    &((struct sockaddr_in *)addrs->ifa_addr)->sin_addr))
+                {
+                    continue;
+                }
+                HostAddress::convertBinaryToText(AF_INET, 
+                    &((struct sockaddr_in *)addrs->ifa_addr)->sin_addr,
+                    buff, sizeof(buff));
+                ips.append(buff);
+                break;
+            case AF_INET6 :
+                if (System::isLoopBack(AF_INET6,
+                    &((struct sockaddr_in6 *)addrs->ifa_addr)->sin6_addr))
+                {
+                    continue;
+                }
+                HostAddress::convertBinaryToText(AF_INET6,
+                    &((struct sockaddr_in6 *)addrs->ifa_addr)->sin6_addr,
+                    buff, sizeof(buff));
+                ips.append(buff);
+                break;
+        }
+    }
+    if(array)
+    {
+        freeifaddrs(array);
+    }
+#elif defined(PEGASUS_OS_AIX)
+//ATTN: implement for AIX
+#elif defined(PEGASUS_OS_ZOS)
+//ATTN: implement for ZOS
+#elif defined(PEGASUS_OS_HPUX)
+//ATTN: implement for HPUX
+#elif defined(PEGASUS_OS_VMS)
+//ATTN: implement for VMS
+#else
+//ATTN: implement for rest of UNIX flavors.
+#endif
+
+#endif // PEGASUS_ENABLE_IPV6
+
+    return ips;
+}
+
 PEGASUS_NAMESPACE_END
