@@ -384,14 +384,16 @@ static int _getEntityRef(char*& p)
     return -1;
 }
 
-static inline int _getCharRef(char*& p, bool hex)
+static inline int _getCharRef(char*& p)
 {
     char* end;
     unsigned long ch;
+    Boolean hex = false;
 
-    if (hex)
+    if (*p == 'x')
     {
-        ch = strtoul(p, &end, 16);
+        hex = true;
+        ch = strtoul(++p, &end, 16);
     }
     else
     {
@@ -413,7 +415,92 @@ static inline int _getCharRef(char*& p, bool hex)
     return ch;
 }
 
-static void _normalize(Uint32& line, char*& p, char end_char, char*& start)
+// Parse an entity reference or a character reference
+static inline int _getRef(Uint32 line, char*& p)
+{
+    int ch;
+
+    if (*p == '#')
+    {
+        ch = _getCharRef(++p);
+    }
+    else
+    {
+        ch = _getEntityRef(p);
+    }
+
+    if (ch == -1)
+    {
+        throw XmlException(XmlException::MALFORMED_REFERENCE, line);
+    }
+
+    return ch;
+}
+
+static inline void _normalizeElementValue(
+    Uint32& line,
+    char*& p)
+{
+    // Process one character at a time:
+
+    char* q = p;
+
+    while (*p && (*p != '<'))
+    {
+        if (_isspace(*p))
+        {
+            // Trim whitespace from the end of the value, but do not compress
+            // whitespace within the value.
+
+            const char* start = p;
+
+            if (*p++ == '\n')
+            {
+                line++;
+            }
+
+            _skipWhitespace(line, p);
+
+            if (*p && (*p != '<'))
+            {
+                // Transfer internal whitespace to q without compressing it.
+                const char* i = start;
+                while (i < p)
+                {
+                    *q++ = *i++;
+                }
+            }
+            else
+            {
+                // Do not transfer trailing whitespace to q.
+                break;
+            }
+        }
+        else if (*p == '&')
+        {
+            // Process an entity reference or a character reference.
+
+            *q++ = _getRef(line, ++p);
+        }
+        else
+        {
+            *q++ = *p++;
+        }
+    }
+
+    // If q got behind p, it is safe and necessary to null-terminate q
+
+    if (q != p)
+    {
+        *q = '\0';
+    }
+}
+
+static inline void _normalizeAttributeValue(
+    Uint32& line,
+    char*& p,
+    char end_char,
+    char*& start)
 {
     // Skip over leading whitespace:
 
@@ -442,36 +529,9 @@ static void _normalize(Uint32& line, char*& p, char end_char, char*& start)
         }
         else if (*p == '&')
         {
-            // Process entity characters and entity references:
+            // Process an entity reference or a character reference.
 
-            p++;
-            int ch;
-
-            if (*p == '#')
-            {
-                *p++;
-
-                if (*p == 'x')
-                {
-                    p++;
-                    ch = _getCharRef(p, true);
-                }
-                else
-                {
-                    ch = _getCharRef(p, false);
-                }
-            }
-            else
-            {
-                ch = _getEntityRef(p);
-            }
-
-            if (ch == -1)
-            {
-                throw XmlException(XmlException::MALFORMED_REFERENCE, line);
-            }
-
-            *q++ = ch;
+            *q++ = _getRef(line, ++p);
         }
         else
         {
@@ -479,16 +539,17 @@ static void _normalize(Uint32& line, char*& p, char end_char, char*& start)
         }
     }
 
-    // We encountered a the end_char or a zero-terminator.
-
-    *q = *p;
-
     // Remove single trailing whitespace (consecutive whitespaces already
     // compressed above).  Since p >= q, we can tell if we need to strip a
     // trailing space from q by looking at the end of p.  We must not look at
     // the last character of p, though, if p is an empty string.
+    Boolean adjust_q = (p != start) && _isspace(p[-1]);
 
-    if ((p != start) && _isspace(p[-1]))
+    // We encountered a the end_char or a zero-terminator.
+
+    *q = *p;
+
+    if (adjust_q)
     {
         q--;
     }
@@ -573,8 +634,8 @@ Boolean XmlParser::next(XmlEntry& entry, Boolean includeComment)
         {
             // Normalize the content:
 
-            char* start;
-            _normalize(_line, _current, '<', start);
+            char* start = _current;
+            _normalizeElementValue(_line, _current);
 
             // Get the content:
 
@@ -886,7 +947,7 @@ void XmlParser::_getElement(char*& p, XmlEntry& entry)
             char quote = *p++;
 
             char* start;
-            _normalize(_line, p, quote, start);
+            _normalizeAttributeValue(_line, p, quote, start);
             attr.value = start;
 
             if (*p != quote)
