@@ -143,6 +143,10 @@ static struct OwnerEntry _properties[] =
 #ifdef PEGASUS_KERBEROS_AUTHENTICATION
     {"kerberosServiceName",
          (ConfigPropertyOwner*)ConfigManager::securityOwner},
+#endif
+#ifdef PEGASUS_OS_ZOS
+    {"enableCFZAPPLID",
+         (ConfigPropertyOwner*)ConfigManager::securityOwner},
 #endif
     {"repositoryIsDefaultInstanceProvider",
          (ConfigPropertyOwner*)ConfigManager::repositoryOwner},
@@ -179,6 +183,8 @@ static struct OwnerEntry _properties[] =
          (ConfigPropertyOwner*)ConfigManager::defaultOwner},
 #endif
     {"socketWriteTimeout",
+         (ConfigPropertyOwner*)ConfigManager::defaultOwner},
+    {"idleConnectionTimeout",
          (ConfigPropertyOwner*)ConfigManager::defaultOwner}
 };
 
