@@ -27,7 +27,7 @@
 // ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 // WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 //
-//==============================================================================
+//=============================================================================
 
 
 ///////////////////////////////////////////////////////////////////////////////
@@ -49,6 +49,7 @@
 
 
 #include <Pegasus/Common/Config.h>
+#include <Pegasus/Common/HostAddress.h>
 #include <Pegasus/Common/PegasusVersion.h>
 
 #include <cctype>
@@ -102,6 +103,8 @@ const CIMName CIMXMLCOMMMECH_PROPERTY_AUTHENTICATIONMECHANISMDESCRIPTIONS(
 const CIMName CIMXMLCOMMMECH_PROPERTY_MULTIPLEOPERATIONSSUPPORTED(
         "MultipleOperationsSupported");
 const CIMName CIMXMLCOMMMECH_PROPERTY_VERSION("Version");
+const CIMName 
+   CIMXMLCOMMMECH_PROPERTY_CIMXMLPROTOVERSION("CIMXMLProtocolVersion");
 const CIMName CIMXMLCOMMMECH_PROPERTY_NAMESPACETYPE("namespaceType");
 const CIMName CIMXMLCOMMMECH_PROPERTY_NAMESPACEACCESSPROTOCOL(
     "namespaceAccessProtocol");
@@ -183,8 +186,13 @@ CIMInstance InteropProvider::buildCIMXMLCommunicationMechanismInstance(
         PEGASUS_CLASSNAME_PG_CIMXMLCOMMUNICATIONMECHANISM.getString());
 
     // Name Property
+    String nameProperty(STRLIT_ARGS("PEGASUSCOMM"));
+    nameProperty.append(namespaceType);
+    nameProperty.append(Char16('+'));
+    nameProperty.append(IPAddress);
+
     setPropertyValue(instance, CIMXMLCOMMMECH_PROPERTY_NAME,
-        (String("PEGASUSCOMM") + namespaceType));
+        nameProperty);
 
     // CommunicationMechanism Property - Force to 2.
     setPropertyValue(instance, CIMXMLCOMMMECH_PROPERTY_COMMUNICATIONMECHANISM,
@@ -227,6 +235,11 @@ CIMInstance InteropProvider::buildCIMXMLCommunicationMechanismInstance(
     setPropertyValue(instance, CIMXMLCOMMMECH_PROPERTY_VERSION,
         CIMXMLProtocolVersion);
 
+    //Populate "required" property that's not populated.
+    // CIMXMLProtocolVersion Property
+    setPropertyValue(instance, CIMXMLCOMMMECH_PROPERTY_CIMXMLPROTOVERSION,
+        Uint16(1));
+
     // NamespaceType Property
     setPropertyValue(instance, CIMXMLCOMMMECH_PROPERTY_NAMESPACETYPE,
         namespaceType);
@@ -314,13 +327,39 @@ Array<CIMInstance> InteropProvider::enumCIMXMLCommunicationMechanismInstances()
             sprintf(buffer, "%u", portNumberHttp);
             httpPort.assign(buffer);
         }
-        CIMInstance instance = 
-            buildCIMXMLCommunicationMechanismInstance(
+        CIMInstance instance;
+        Array<String> ips;
+#ifdef PEGASUS_ENABLE_IPV6
+        ips = System::getInterfaceAddrs();
+        for (Uint32 i = 0; i < ips.size() ; ++i)
+        {
+            String addr = ips[i];
+            if (HostAddress::isValidIPV6Address(ips[i]))
+            {
+                addr = "[" + addr + "]";
+            }
+            addr.append(":");
+            addr.append(httpPort);
+            
+            instance  =
+                buildCIMXMLCommunicationMechanismInstance(
+                    namespaceType,
+                    namespaceAccessProtocol,
+                    addr,
+                    commMechClass);
+            instances.append(instance);
+        }
+#endif
+        // If System::getInterfaceAddrs() fails add ip4 addr here.
+        if (!ips.size())
+        {
+            instance  = buildCIMXMLCommunicationMechanismInstance(
                 namespaceType,
                 namespaceAccessProtocol,
                 getHostAddress(hostName, namespaceAccessProtocol, httpPort),
                 commMechClass);
-        instances.append(instance);
+            instances.append(instance);
+        }
     }
 
     if (enableHttpsConnection)
@@ -337,14 +376,38 @@ Array<CIMInstance> InteropProvider::enumCIMXMLCommunicationMechanismInstances()
             sprintf(buffer, "%u", portNumberHttps);
             httpsPort.assign(buffer);
         }
-        CIMInstance instance = 
-            buildCIMXMLCommunicationMechanismInstance(
+        CIMInstance instance;
+        Array<String> ips;
+#ifdef PEGASUS_ENABLE_IPV6
+        ips = System::getInterfaceAddrs();
+        for (Uint32 i = 0; i < ips.size() ; ++i)
+        {
+            String addr = ips[i];
+            if (HostAddress::isValidIPV6Address(ips[i]))
+            {
+                addr = "[" + addr + "]";
+            }
+            addr.append(":");  
+            addr.append(httpsPort);  
+            instance  =
+                buildCIMXMLCommunicationMechanismInstance(
+                    namespaceType,
+                    namespaceAccessProtocol,
+                    addr,
+                    commMechClass);
+            instances.append(instance);
+        }
+#endif
+        // If System::getInterfaceAddrs() fails add ip4 addr here.
+        if (!ips.size())
+        {
+            instance  = buildCIMXMLCommunicationMechanismInstance(
                 namespaceType,
                 namespaceAccessProtocol,
                 getHostAddress(hostName, namespaceAccessProtocol, httpsPort),
                 commMechClass);
-
-        instances.append(instance);
+            instances.append(instance);
+        }
     }
 
     PEG_METHOD_EXIT();
