@@ -220,6 +220,7 @@ void ProviderRegistrationProvider::modifyInstance(
         userName = String::EMPTY;
     }
 
+#ifndef PEGASUS_OS_ZOS
     if ((userName != String::EMPTY) && !System::isPrivilegedUser(userName))
     {
         throw PEGASUS_CIM_EXCEPTION_L(CIM_ERR_ACCESS_DENIED,MessageLoaderParms(
@@ -229,6 +230,7 @@ void ProviderRegistrationProvider::modifyInstance(
             "You must have superuser privilege to modify the"
                 " registration."));      
     }
+#endif
 
     if(!instanceReference.getNameSpace().equal (PEGASUS_NAMESPACENAME_INTEROP))
     {
@@ -301,6 +303,7 @@ void ProviderRegistrationProvider::createInstance(
         userName = String::EMPTY;
     }
 
+#ifndef PEGASUS_OS_ZOS
     if ((userName != String::EMPTY) && !System::isPrivilegedUser(userName))
     {
         throw PEGASUS_CIM_EXCEPTION_L(CIM_ERR_ACCESS_DENIED, MessageLoaderParms(
@@ -309,6 +312,7 @@ void ProviderRegistrationProvider::createInstance(
                 "SUPERUSER_PRIVILEGE_REQUIRED_REGISTER_PROVIDERS",
             "You must have superuser privilege to register providers."));   
     }
+#endif
 
     CIMName className = instanceReference.getClassName();
     CIMNamespaceName nameSpace = instanceReference.getNameSpace();
@@ -499,9 +503,11 @@ void ProviderRegistrationProvider::createInstance(
             }
 #else
             if (!((userContext == PG_PROVMODULE_USERCTXT_REQUESTOR) ||
-                  (userContext == PG_PROVMODULE_USERCTXT_DESIGNATED) ||
+#ifndef PEGASUS_OS_ZOS
+                  (userContext == PG_PROVMODULE_USERCTXT_CIMSERVER) ||
                   (userContext == PG_PROVMODULE_USERCTXT_PRIVILEGED) ||
-                  (userContext == PG_PROVMODULE_USERCTXT_CIMSERVER)))
+#endif
+                  (userContext == PG_PROVMODULE_USERCTXT_DESIGNATED)))
             {
                 throw PEGASUS_CIM_EXCEPTION_L(CIM_ERR_NOT_SUPPORTED,
                     MessageLoaderParms(
@@ -795,6 +801,7 @@ void ProviderRegistrationProvider::deleteInstance(
         userName = String::EMPTY;
     }
 
+#ifndef PEGASUS_OS_ZOS
     if ((userName != String::EMPTY) && !System::isPrivilegedUser(userName))
     {
         throw PEGASUS_CIM_EXCEPTION_L(CIM_ERR_ACCESS_DENIED,MessageLoaderParms(
@@ -803,6 +810,7 @@ void ProviderRegistrationProvider::deleteInstance(
                 "SUPERUSER_PRIVILEGE_REQUIRED_UNREGISTER_PROVIDERS",
             "You must have superuser privilege to unregister providers."));     
     }
+#endif
 
     if(!instanceReference.getNameSpace().equal (PEGASUS_NAMESPACENAME_INTEROP))
     {
@@ -992,6 +1000,7 @@ void ProviderRegistrationProvider::invokeMethod(
         userName = String::EMPTY;
     }
 
+#ifndef PEGASUS_OS_ZOS
     if ((userName != String::EMPTY) && !System::isPrivilegedUser(userName))
     {
         throw PEGASUS_CIM_EXCEPTION_L(CIM_ERR_ACCESS_DENIED,
@@ -1002,6 +1011,7 @@ void ProviderRegistrationProvider::invokeMethod(
             "You must have superuser privilege to disable or enable"
                 " providers."));      
     }
+#endif
 
     if(!objectReference.getNameSpace().equal (PEGASUS_NAMESPACENAME_INTEROP))
     {
