@@ -39,6 +39,7 @@
 #include <Pegasus/Common/PegasusVersion.h>
 
 #include <Pegasus/Common/MessageLoader.h>
+#include <Pegasus/Config/ConfigManager.h>
 
 #include <Pegasus/ProviderManager2/CMPI/CMPIProvider.h>
 #include <Pegasus/ProviderManager2/CMPI/CMPIProviderModule.h>
@@ -47,9 +48,6 @@
 PEGASUS_USING_STD;
 PEGASUS_NAMESPACE_BEGIN
 
-#undef IDLE_LIMIT
-#define IDLE_LIMIT 50
-
 /* Thread deletion specific */
 Semaphore CMPILocalProviderManager::_pollingSem(0);
 AtomicInt CMPILocalProviderManager::_stopPolling(0);
@@ -59,7 +57,7 @@ List<CMPILocalProviderManager::cleanupThreadRecord,Mutex>
 Mutex CMPILocalProviderManager::_reaperMutex;
 
 CMPILocalProviderManager::CMPILocalProviderManager ():
-_idle_timeout (IDLE_LIMIT)
+_idle_timeout (PEGASUS_PROVIDER_IDLE_TIMEOUT_SECONDS)
 {
     PEG_METHOD_ENTER(
         TRC_PROVIDERMANAGER,
@@ -125,8 +123,8 @@ Sint32 CMPILocalProviderManager::_provider_ctrl (
                 String location = *(parms->location);
 
                 CMPIProvider *pr = 0;
-                CMPIProvider::OpProviderHolder * ph =
-                    reinterpret_cast < CMPIProvider::OpProviderHolder * >(ret);
+                OpProviderHolder * ph =
+                    reinterpret_cast < OpProviderHolder * >(ret);
 
                 pr = _lookupProvider (providerName);
 
@@ -169,26 +167,26 @@ Sint32 CMPILocalProviderManager::_provider_ctrl (
                     UNLOAD_PROVIDER");
                 CMPIProvider *pr = 0;
                 pr = _lookupProvider (*(parms->providerName));
+
+                // The provider table must be locked before unloading.
+                AutoMutex lock (_providerTableMutex);
                 if ((pr->getStatus () == CMPIProvider::INITIALIZED))
                 {
-
                     PEG_TRACE_STRING(
                         TRC_PROVIDERMANAGER,
                         Tracer::LEVEL4,
                         "Unloading CMPIProvider: " + pr->getName());
-
-                    AutoMutex lock (_providerTableMutex);
-                    // The provider table must be locked before unloading. 
-                    _providers.remove (pr->_name);
-                    _unloadProvider (pr);
+                    _unloadProvider(pr);
+                }
+                if (pr->getStatus () == CMPIProvider::UNINITIALIZED)
+                {
+                    _providers.remove (pr->getNameWithType());
                     delete pr;
-
                 }
                 else
                 {
-                    // No need to have a memory leak.
-                    _providers.remove(pr->_name);
-                    delete pr;
+                    // Provider could not be unloaded due to pending operations.
+                    ccode = -1;
                 }
                 break;
             }
@@ -299,6 +297,7 @@ Sint32 CMPILocalProviderManager::_provider_ctrl (
                 CMPIProvider *provider = 0;
                 // Locked provider mutex.
                 AutoMutex lock (_providerTableMutex);
+                Array<CMPIProvider*> unloadPendingProviders;
 
                 PEG_TRACE((
                     TRC_PROVIDERMANAGER,
@@ -322,9 +321,23 @@ Sint32 CMPILocalProviderManager::_provider_ctrl (
                         else
                         {
                             _unloadProvider (provider);
-                            delete provider;
+                            if (provider->getStatus () ==
+                                CMPIProvider::UNINITIALIZED)
+                            {
+                                delete provider;
+                            }
+                            else
+                            {
+                                unloadPendingProviders.append(provider);
+                            }
                         }
                     }
+
+                    if (unloadPendingProviders.size())
+                    {
+                        _terminateUnloadPendingProviders(
+                            unloadPendingProviders);
+                    }
                     // All the providers are removed. Clear the hash-table
                     _providers.clear ();
                 }
@@ -378,14 +391,14 @@ Sint32 CMPILocalProviderManager::_provider_ctrl (
                                 continue;
                             }
 
-                            if (provider->_quantum == quantum)
+                            if (provider->getQuantum() == quantum)
                             {
                                 continue;
                             }
 
-                            provider->_quantum = quantum;
+                            provider->setQuantum(quantum);
 
-                            if (provider->_current_operations.get ())
+                            if (provider->getCurrentOperations())
                             {
                                 PEG_TRACE_STRING(
                                     TRC_PROVIDERMANAGER,
@@ -437,7 +450,7 @@ Sint32 CMPILocalProviderManager::_provider_ctrl (
                             {
                                 // lock the provider mutex
 
-                                AutoMutex pr_lock (provider->_statusMutex);
+                                AutoMutex pr_lock (provider->getStatusMutex());
 
                                 if (provider->tryTerminate () == false)
                                 {
@@ -451,10 +464,10 @@ Sint32 CMPILocalProviderManager::_provider_ctrl (
                                     continue;
                                 }
 
-                                PEGASUS_ASSERT (provider->_module != 0);
+                                PEGASUS_ASSERT (provider->getModule() != 0);
 
                                 // unload provider module
-                                provider->_module->unloadModule ();
+                                provider->getModule()->unloadModule ();
                                 Logger::put (Logger::STANDARD_LOG, 
                                     System::CIMSERVER,
                                     Logger::TRACE,
@@ -473,7 +486,7 @@ Sint32 CMPILocalProviderManager::_provider_ctrl (
                                     Tracer::LEVEL4,
                                     "Destroying CMPIProvider's CIMOM Handle "
                                     + provider->getName());
-                                delete provider->_cimom_handle;
+                                delete provider->getCIMOMHandle();
 
                                 // set provider status to UNINITIALIZED
                                 provider->reset ();
@@ -545,6 +558,27 @@ ThreadReturnType PEGASUS_THREAD_CDECL CMPILocalProviderManager::_reaper(
     PEG_METHOD_EXIT();
     return(0);
 }
+
+Boolean CMPILocalProviderManager::isProviderActive(const String &providerName)
+{
+    PEG_METHOD_ENTER(
+        TRC_PROVIDERMANAGER,
+        "CMPILocalProviderManager::isProviderActive()");
+
+    String lProviderName("L");
+    lProviderName.append(providerName);
+    String rProviderName("R");
+    rProviderName.append(providerName);
+
+    AutoMutex mtx(_providerTableMutex);
+    Boolean active = _providers.contains(lProviderName) ||
+        _providers.contains(rProviderName);
+
+    PEG_METHOD_EXIT();
+
+    return active;
+}
+
 /*
  // Cleanup the thread and upon deletion of it, call the CMPIProvider' 
  // "threadDeleted". to not, all the CMPIProvider '
@@ -601,10 +635,10 @@ void CMPILocalProviderManager::cleanupThread(Thread *t, CMPIProvider *p)
 
 }
 
-CMPIProvider::OpProviderHolder CMPILocalProviderManager::
+OpProviderHolder CMPILocalProviderManager::
     getRemoteProvider (const String & location, const String & providerName)
 {
-    CMPIProvider::OpProviderHolder ph;
+    OpProviderHolder ph;
     CTRL_STRINGS
         strings;
     Sint32
@@ -652,10 +686,10 @@ CMPIProvider::OpProviderHolder CMPILocalProviderManager::
     return ph;
 }
 
-CMPIProvider::OpProviderHolder CMPILocalProviderManager::
+OpProviderHolder CMPILocalProviderManager::
     getProvider (const String & fileName, const String & providerName)
 {
-    CMPIProvider::OpProviderHolder ph;
+    OpProviderHolder ph;
     CTRL_STRINGS strings;
     Sint32 ccode;
 
@@ -706,7 +740,7 @@ CMPIProvider::OpProviderHolder CMPILocalProviderManager::
 
 }
 
-void CMPILocalProviderManager::unloadProvider(
+Boolean CMPILocalProviderManager::unloadProvider(
     const String & fileName,
     const String & providerName)
 {
@@ -721,13 +755,17 @@ void CMPILocalProviderManager::unloadProvider(
     strings.fileName = &fileName;
     strings.providerName = &lproviderName;
     strings.location = &String::EMPTY;
-    _provider_ctrl (UNLOAD_PROVIDER, &strings, (void *) 0);
+
+    int lproviderStatus = 0;
+    int rproviderStatus = 0;
+    lproviderStatus = _provider_ctrl (UNLOAD_PROVIDER, &strings, (void *) 0);
 
     strings.providerName = &rproviderName;
 
-    _provider_ctrl (UNLOAD_PROVIDER, &strings, (void *) 0);
+    rproviderStatus = _provider_ctrl (UNLOAD_PROVIDER, &strings, (void *) 0);
 
     PEG_METHOD_EXIT ();
+    return lproviderStatus != -1 && rproviderStatus != -1;
 }
 
 void CMPILocalProviderManager::shutdownAllProviders ()
@@ -890,9 +928,9 @@ CMPIProvider *CMPILocalProviderManager::_initProvider(
     String exceptionMsg = moduleFileName;
     {
         // lock the provider status mutex
-        AutoMutex lock (provider->_statusMutex);
+        AutoMutex lock (provider->getStatusMutex());
 
-        if (provider->_status == CMPIProvider::INITIALIZED)
+        if (provider->getStatus() == CMPIProvider::INITIALIZED)
         {
             PEG_METHOD_EXIT();
             // Initialization is already complete
@@ -907,7 +945,7 @@ CMPIProvider *CMPILocalProviderManager::_initProvider(
         // load the provider
         try
         {
-            base = module->load (provider->_name);
+            base = module->load (provider->getNameWithType());
             moduleLoaded = true;
         }
         catch (const Exception &e)
@@ -941,11 +979,11 @@ CMPIProvider *CMPILocalProviderManager::_initProvider(
 
             CIMOMHandle *cimomHandle = new CIMOMHandle ();
             provider->set (module, base, cimomHandle);
-            provider->_quantum = 0;
+            provider->setQuantum(0);
 
             try
             {
-                provider->initialize (*(provider->_cimom_handle));
+                provider->initialize (*(provider->getCIMOMHandle()));
             }
             catch (const Exception &e)
             {
@@ -984,12 +1022,12 @@ CMPIProvider *CMPILocalProviderManager::_initProvider(
         }
 
         // delete the cimom handle
-        delete provider->_cimom_handle;
+        delete provider->getCIMOMHandle();
         // set provider status to UNINITIALIZED
         provider->reset ();
 
         AutoMutex lock (_providerTableMutex);
-        _providers.remove (provider->_name);
+        _providers.remove (provider->getNameWithType());
         delete provider;
 
         PEG_METHOD_EXIT ();
@@ -1000,8 +1038,79 @@ CMPIProvider *CMPILocalProviderManager::_initProvider(
     return(provider);
 }
 
+/*
+    This method is called when CMPIProviderManager receives  
+    CIMStopAllProvidersRequestMessage and 
+    CMPILocalProviderManager::_provider_ctrl() method could not unload the 
+    provider(s) because of pending requests with the provider. We give grace
+    time of (shutdownTimeout - 1) seconds to the unload pending providers
+    to unload gracefully before terminating them forcibly. Note that this
+    happens only when provider is running out-of-process and communication
+    with CIMServer falied because of pipe read/write failures.
+*/
+void CMPILocalProviderManager::_terminateUnloadPendingProviders(
+    Array<CMPIProvider*> &unloadPendingProviders)
+{
+    PEG_METHOD_ENTER(
+        TRC_PROVIDERMANAGER,
+        "CMPILocalProviderManager::_terminateUnloadPendingProviders()");
 
-void CMPILocalProviderManager::_unloadProvider (CMPIProvider * provider)
+    PEG_TRACE((
+        TRC_PROVIDERMANAGER,
+        Tracer::LEVEL3,
+        "Unloading %u unload-pending providers.",
+        unloadPendingProviders.size()));
+
+    String configTimeout =
+        ConfigManager::getInstance()->getCurrentValue("shutdownTimeout");
+
+    Uint32 timeoutValue =
+        strtol(configTimeout.getCString(), (char **)0, 10);
+
+    for (Uint32 waitTime = timeoutValue - 1; waitTime > 0; waitTime--)
+    {
+        Boolean unloadPending = false;
+        for (Uint32 j = 0, n = unloadPendingProviders.size(); j < n; ++j)
+        {
+            if (unloadPendingProviders[j]->getStatus() == 
+                CMPIProvider::INITIALIZED)
+            {
+                _unloadProvider(unloadPendingProviders[j]);
+                if (unloadPendingProviders[j]->getStatus()
+                    == CMPIProvider::INITIALIZED)
+                {
+                    unloadPending = true;
+                }
+            }
+        }
+        if (!unloadPending)
+        {
+            break;
+        }
+        Threads::sleep(1000);
+    }
+
+    for (Uint32 j = 0, n = unloadPendingProviders.size(); j < n; ++j)
+    {
+        if (unloadPendingProviders[j]->getStatus() ==
+            CMPIProvider::INITIALIZED)
+        {
+            // Force unload
+            _unloadProvider(unloadPendingProviders[j], true);    
+        }
+        if (unloadPendingProviders[j]->getStatus() ==
+            CMPIProvider::UNINITIALIZED)
+        {
+            delete unloadPendingProviders[j];
+        }
+    }
+
+    PEG_METHOD_EXIT();
+}
+
+void CMPILocalProviderManager::_unloadProvider (
+    CMPIProvider * provider,
+    Boolean forceUnload)
 {
     //
     // NOTE:  It is the caller's responsibility to make sure that
@@ -1016,7 +1125,7 @@ void CMPILocalProviderManager::_unloadProvider (CMPIProvider * provider)
         Tracer::LEVEL4,
         "Unloading Provider " + provider->getName());
 
-    if (provider->_current_operations.get ())
+    if (provider->getCurrentOperations() && !forceUnload)
     {
         PEG_TRACE_STRING(
             TRC_PROVIDERMANAGER,
@@ -1026,14 +1135,24 @@ void CMPILocalProviderManager::_unloadProvider (CMPIProvider * provider)
     }
     else
     {
-        PEG_TRACE_STRING(
-            TRC_PROVIDERMANAGER,
-            Tracer::LEVEL4,
-            "Terminating Provider " + provider->getName());
-
+        if (provider->getCurrentOperations())
+        { 
+            PEG_TRACE_STRING(
+                TRC_PROVIDERMANAGER,
+                Tracer::LEVEL2,
+                "Terminating Provider with pending operations " +
+                    provider->getName());
+        }
+        else
+        {
+            PEG_TRACE_STRING(
+                TRC_PROVIDERMANAGER,
+                Tracer::LEVEL4,
+                "Terminating Provider " + provider->getName());
+         }
 
         // lock the provider mutex
-        AutoMutex pr_lock (provider->_statusMutex);
+        AutoMutex pr_lock (provider->getStatusMutex());
 
         try
         {
@@ -1048,27 +1167,33 @@ void CMPILocalProviderManager::_unloadProvider (CMPIProvider * provider)
                 provider->getName());
         }
 
-        // delete the cimom handle
-        PEG_TRACE_STRING(
-            TRC_PROVIDERMANAGER,
-            Tracer::LEVEL4,
-            "Destroying CMPIProvider's CIMOM Handle " +
-            provider->getName());
+        if (provider->getStatus() == CMPIProvider::UNINITIALIZED)
+        {
+            // unload provider module
+            PEGASUS_ASSERT (provider->getModule() != 0);
+            provider->getModule()->unloadModule ();
+            Logger::put (
+                Logger::STANDARD_LOG,
+                System::CIMSERVER,
+                Logger::TRACE,
+                "CMPILocalProviderManager::_provider_crtl -  "
+                    "Unload provider $0",
+                provider->getName ());
 
-        delete provider->_cimom_handle;
-        PEGASUS_ASSERT (provider->_module != 0);
+            // delete the cimom handle
+            PEG_TRACE_STRING(
+                TRC_PROVIDERMANAGER,
+                Tracer::LEVEL4,
+                "Destroying CMPIProvider's CIMOM Handle " +
+                provider->getName());
+            delete provider->getCIMOMHandle();
 
-        // unload provider module
-        provider->_module->unloadModule ();
-        Logger::put (Logger::STANDARD_LOG, System::CIMSERVER, Logger::TRACE,
-            "CMPILocalProviderManager::_provider_crtl -  Unload provider $0",
-            provider->getName ());
+            // set provider status to UNINITIALIZED
+            provider->reset ();
 
-        // set provider status to UNINITIALIZED
-        provider->reset ();
-
-        // Do not delete the provider. The function calling this function
-        // takes care of that.
+            // Do not delete the provider. The function calling this function
+            // takes care of that.
+        }
     }
 
     PEG_METHOD_EXIT ();
