@@ -103,7 +103,7 @@ CMPIStatus resolveEmbeddedInstanceTypes(
                   with the relevant class definitions.
                 */
                 CIMProperty normalizedProperty(
-                    ObjectNormalizer::_processProperty(
+                    ObjectNormalizer::processProperty(
                     propertyDef,
                     currentProp,
                     false,
@@ -301,45 +301,16 @@ extern "C"
             CIMInstance& inst=*(CIMInstance*)(eInst->hdl);
             CMPI_Result *xRes=(CMPI_Result*)eRes;
             const CIMObjectPath& op=inst.getPath();
-            CIMClass *cc=mbGetClass(xRes->xBroker,op);
-            CIMObjectPath iop=inst.buildPath(*cc);
-            iop.setNameSpace(op.getNameSpace());
-            inst.setPath(iop);
-
-            /**
-                Check if a property filter has been set. If yes throw out
-               all properties which are not found in the filter list.
-            */
-            char **listroot=(char**)
-                           (reinterpret_cast<const CMPI_Object*>(eInst))->priv;
-
-            if (listroot && *listroot)
+            //Build objectpath if keybindings are not found. This will happen
+            //when this instance is created with empty ObjectPath and Provider
+            //did not set objectpath in the instance.
+            if (op.getKeyBindings().size() == 0)
             {
-                int propertyCount = inst.getPropertyCount()-1;
-                for (int idx=propertyCount; idx >=0 ; idx--)
-                {
-                    CIMConstProperty prop = inst.getProperty(idx);
-                    String sName = prop.getName().getString();
-                    char * name = strdup(sName.getCString());
-                    char **list = listroot;
-                    int found = false;
-                    while (*list)
-                    {
-                        if (System::strcasecmp(name,*list)==0)
-                        {
-                            found = true;
-                            break;
-                        }
-                        list++;
-                    }
-                    free(name);
-                    if (!found)
-                    {
-                        inst.removeProperty(idx);
-                    }
-                }
+                CIMClass *cc=mbGetClass(xRes->xBroker,op);
+                CIMObjectPath iop=inst.buildPath(*cc);
+                iop.setNameSpace(op.getNameSpace());
+                inst.setPath(iop);
             }
-
 #ifdef PEGASUS_EMBEDDED_INSTANCE_SUPPORT
             /**
               CMPI does not differentiate between EmbeddedInstances and
@@ -427,10 +398,16 @@ extern "C"
             CIMInstance& inst=*(CIMInstance*)(eInst->hdl);
             CMPI_Result *xRes=(CMPI_Result*)eRes;
             const CIMObjectPath& op=inst.getPath();
-            CIMClass *cc=mbGetClass(xRes->xBroker,op);
-            CIMObjectPath iop=inst.buildPath(*cc);
-            iop.setNameSpace(op.getNameSpace());
-            inst.setPath(iop);
+            //Build objectpath if keybindings are not found. This will happen
+            //when this instance is created with empty ObjectPath and Provider
+            //did not set objectpath in the instance.
+            if (op.getKeyBindings().size() == 0)
+            {
+                CIMClass *cc=mbGetClass(xRes->xBroker,op);
+                CIMObjectPath iop=inst.buildPath(*cc);
+                iop.setNameSpace(op.getNameSpace());
+                inst.setPath(iop);
+            }
 
 #ifdef PEGASUS_EMBEDDED_INSTANCE_SUPPORT
            /**
@@ -556,40 +533,6 @@ extern "C"
             }
             CIMInstance& inst=*(CIMInstance*)(eInst->hdl);
             CMPI_Result *xRes=(CMPI_Result*)eRes;
-            /**
-              Check if a property filter has been set. If yes throw out
-             all properties which are not found in the filter list.
-           */
-            char **listroot=(char**)
-                           (reinterpret_cast<const CMPI_Object*>(eInst))->priv;
-
-            if (listroot && *listroot)
-            {
-                int propertyCount = inst.getPropertyCount()-1;
-                for (int idx=propertyCount; idx >=0 ; idx--)
-                {
-                    CIMConstProperty prop = inst.getProperty(idx);
-                    String sName = prop.getName().getString();
-                    char * name = strdup(sName.getCString());
-                    char **list = listroot;
-                    int found = false;
-                    while (*list)
-                    {
-                        if (System::strcasecmp(name,*list)==0)
-                        {
-                            found = true;
-                            break;
-                        }
-                        list++;
-                    }
-                    free(name);
-                    if (!found)
-                    {
-                        inst.removeProperty(idx);
-                    }
-                }
-            }
-
             res->deliver(inst);
         }
         catch (const CIMException &e)
