@@ -107,6 +107,7 @@ struct InheritanceTreeExt
 struct InheritanceTreeNode
 {
     InheritanceTreeNode(const CIMName& className);
+    ~InheritanceTreeNode();
 
     void addSubClass(InheritanceTreeNode* subClass);
 
@@ -140,6 +141,18 @@ InheritanceTreeNode::InheritanceTreeNode(const CIMName& className)
 {
 }
 
+InheritanceTreeNode::~InheritanceTreeNode()
+{
+    if (extension)
+    {
+        for(Uint32 i = 0, size = extNodes->size(); i < size; i++)
+        {
+            delete (*extNodes)[i];
+        }
+        delete extNodes;
+    }
+}
+
 void InheritanceTreeNode::addSubClass(InheritanceTreeNode* subClass)
 {
     subClass->superClass = this;
@@ -307,16 +320,6 @@ void InheritanceTree::insert(
     extNode->extNodes->append(new InheritanceTreeExt(tag,classNode));
 
     classNode->superClass = superClassNode;
-    /* temp comment out this code from bug 3352.  See bug 3498 for reason
-    if (extNode)
-    {
-       for (int i=0, m=extNode->extNodes->size(); i < m; i++)
-          if ((*extNode->extNodes)[i])
-            delete (*(extNode->extNodes))[i];
-       delete extNode;
-    }
-    extNode = NULL;*/
-
 }
 
 void InheritanceTree::insert(
