@@ -219,6 +219,13 @@ CIMServer::CIMServer()
     _cimserver = this;
     _init();
 
+    // Get value of idle connection timeout in seconds.
+    String idleConnectionConfigTimeout =
+        ConfigManager::getInstance()->getCurrentValue("idleConnectionTimeout");
+
+    _idleConnectionTimeoutSeconds =
+        strtol(idleConnectionConfigTimeout.getCString(), (char**)0, 10);
+ 
     PEG_METHOD_EXIT();
 }
 
@@ -501,6 +508,13 @@ void CIMServer::_init()
     getSigHandle()->registerHandler(SIGIO, _asynchronousSignalHandler);
     getSigHandle()->activate(SIGIO);
 #endif
+#ifdef PEGASUS_OS_ZOS
+    // Establish handling signal send to us on USS shutdown
+    getSigHandle()->registerHandler(PEGASUS_SIGDANGER, shutdownSignalHandler);
+    getSigHandle()->activate(PEGASUS_SIGDANGER);
+    // enable process to receive SIGDANGER on USS shutdown
+    __shutdown_registration(_SDR_NOTIFY, _SDR_REGPROCESS, _SDR_SENDSIGDANGER);
+#endif
 
     //
     // Set up an additional thread waiting for commands from the
@@ -657,6 +671,9 @@ void CIMServer::addAcceptor(
         socketWriteTimeout = PEGASUS_DEFAULT_SOCKETWRITE_TIMEOUT_SECONDS;
     acceptor->setSocketWriteTimeout(socketWriteTimeout);
 
+    // Set idle connection timeout in seconds.
+    acceptor->setIdleConnectionTimeout(_idleConnectionTimeoutSeconds);
+
     _acceptors.append(acceptor);
 }
 
@@ -1320,7 +1337,8 @@ ThreadReturnType PEGASUS_THREAD_CDECL registerPegasusWithSLP(void* parm)
                                                  "DSA",
                                                  scopes,
                                                  FALSE,
-                                                 FALSE)))
+                                                 FALSE,
+                                                 0)))
          {
              if (foundHttpProtocol &&
                  (!client->srv_reg_local(client, 
@@ -1431,7 +1449,8 @@ void PEGASUS_SERVER_LINKAGE unregisterPegasusFromSLP()
                                             "DSA",
                                             scopes,
                                             FALSE,
-                                            FALSE)))
+                                            FALSE,
+                                            0)))
     {
         if (foundHttpProtocol)
         {
