@@ -45,7 +45,8 @@
 //
 // cimserver daemon=false
 //
-// The daemon config property has no effect on windows operation.
+// The daemon config property has no effect on windows operation or when
+// privilege separation is enabled.
 //
 // To shutdown pegasus, use the -s option:
 //
@@ -508,14 +509,16 @@ int main(int argc, char** argv)
         for (int i = 1; i < argc; )
         {
             const char* arg = argv[i];
-            if (String::equal(arg,"--help"))
+            if (strcmp(arg, "--help") == 0)
             {
-                    PrintHelp(argv[0]);
-                    exit(0);
+                PrintHelp(argv[0]);
+                Executor::daemonizeExecutor();
+                exit(0);
             }
-            else if (String::equal(arg,"--version"))
+            else if (strcmp(arg, "--version") == 0)
             {
                 cout << _cimServerProcess->getCompleteVersion() << endl;
+                Executor::daemonizeExecutor();
                 exit(0);
             }
             // Check for -option
@@ -531,6 +534,7 @@ int main(int argc, char** argv)
                     strlen(option) == 1)
                 {
                     cout << _cimServerProcess->getCompleteVersion() << endl;
+                    Executor::daemonizeExecutor();
                     exit(0);
                 }
                 //
@@ -540,6 +544,7 @@ int main(int argc, char** argv)
                         (strlen(option) == 1))
                 {
                     PrintHelp(argv[0]);
+                    Executor::daemonizeExecutor();
                     exit(0);
                 }
 #if !defined(PEGASUS_USE_RELEASE_DIRS)
@@ -558,7 +563,7 @@ int main(int argc, char** argv)
                             "Missing argument for option -$0",
                             opt);
                         cout << MessageLoader::getMessage(parms) << endl;
-                        exit(0);
+                        exit(1);
                     }
 
                     memmove(&argv[i], &argv[i + 2], (argc-i-1) * sizeof(char*));
@@ -602,7 +607,7 @@ int main(int argc, char** argv)
                             "Duplicate shutdown option specified.");
 
                         cout << MessageLoader::getMessage(parms) << endl;
-                        exit(0);
+                        exit(1);
                     }
 
                     shutdownOption = true;
@@ -766,6 +771,16 @@ int CIMServerProcess::cimserver_run(
         daemonOption = ConfigManager::parseBooleanValue(
             configManager->getCurrentValue("daemon"));
 
+        if ((Executor::detectExecutor() == 0) && (daemonOption == false))
+        {
+            MessageLoaderParms parms(
+                "src.Server.cimserver.PRIVSEP_REQUIRES_DAEMON",
+                "Warning: The configuration setting daemon=false is ignored "
+                    "with privilege separation enabled.");
+            cerr << MessageLoader::getMessage(parms) << endl;
+            daemonOption = true;
+        }
+
 #if !defined(PEGASUS_USE_SYSLOGS)
         String logsDirectory = ConfigManager::getHomedPath(
             configManager->getCurrentValue("logdir"));
@@ -1112,11 +1127,7 @@ int CIMServerProcess::cimserver_run(
         // so user knows that there is cimserver ready to serve CIM requests.
         if (daemonOption)
         {
-#if defined(PEGASUS_ENABLE_PRIVILEGE_SEPARATION)
-            Executor::daemonizeExecutor();
-#else
             _cimServerProcess->notify_parent(0);
-#endif
         }
 
 #if defined(PEGASUS_DEBUG)
